import javax.swing.*;
import java.awt.*;
import java.util.List;
import java.util.*;

/**
 * Canvas est une classe qui permet un dessin graphique simple sur un fond.
 * Elle modifie la classe Canvas, plus gnraliste, spcifiquement conue pour
 * l'exemple "shapes" de BlueJ. 
 *
 * @author: Bruce Quig
 * @author: Michael Kolling (mik)
 *
 * @version: 1.6 (shapes)
 */
public class Canvas
{
    // Note : l'implantation de cette classe (spcifiquement la gestion de
    // l'identit et des couleurs des formes) est lgrement plus complexe que ncessaire.
    // Ceci est fait  dessein pour s'assurer que l'interface et les champs d'instance
    // des objets shape de ce projet restent simples, pour apprendre facilement.

    private static Canvas canvasSingleton;

    /**
     * Mthode permettant d'obtenir le seul objet du fond.
     */
    public static Canvas getCanvas()
    {
        if(canvasSingleton == null) {
            canvasSingleton = new Canvas("Dmo Shapes de BlueJ", 300, 300, 
                                         Color.white);
        }
        canvasSingleton.setVisible(true);
        return canvasSingleton;
    }

    //  ----- partie instance -----

    private JFrame frame;
    private CanvasPane canvas;
    private Graphics2D graphic;
    private Color backgroundColour;
    private Image canvasImage;
    private List objects;
    private HashMap shapes;
    
    /**
     * Cre un Canvas.
     * @param title  Le titre qui apparatra dans le cadre
     * @param width  La largeur souhaite pour le fond
     * @param height  La hauteur souhaite pour le fond
     * @param bgClour  La couleur d'arrire-plan souhaite pour le fond
     */
    private Canvas(String title, int width, int height, Color bgColour)
    {
        frame = new JFrame();
        canvas = new CanvasPane();
        frame.setContentPane(canvas);
        frame.setTitle(title);
        canvas.setPreferredSize(new Dimension(width, height));
        backgroundColour = bgColour;
        frame.pack();
        objects = new ArrayList();
        shapes = new HashMap();
    }

    /**
     * Dfinit la visibilit du fond et amne le fond sur le devant de l'cran
     * lorsqu'il apparat. Cette mthode permet aussi d'amener un fond dj 
     * visible  l'avant d'autres fentres.
     * @param visible  Valeur boolenne reprsentant la visibilit souhaite du
     * fond (true ou false) 
     */
    public void setVisible(boolean visible)
    {
        if(graphic == null) {
            // premire fois : instantie l'image hors cran et la remplit 
            // de la couleur d'arrire-plan
            Dimension size = canvas.getSize();
            canvasImage = canvas.createImage(size.width, size.height);
            graphic = (Graphics2D)canvasImage.getGraphics();
            graphic.setColor(backgroundColour);
            graphic.fillRect(0, 0, size.width, size.height);
            graphic.setColor(Color.black);
        }
        frame.setVisible(visible);
    }

    /**
     * Dessine une forme donne sur le fond.
     * @param  referenceObject  Un objet pour dfinir l'identit de cette forme
     * @param  color            La couleur de la forme
     * @param  shape            L'objet forme  dessiner sur le fond
     */
     // Note : ceci est une mthode un peu basique pour grer les objets
     // shape. Elle est soigneusement conue pour assurer la bonne tenue des interfaces de la forme visible
     // de ce projet pour des raisons d'enseignement.
    public void draw(Object referenceObject, String color, Shape shape)
    {
        objects.remove(referenceObject);   // au cas o il aurait dj t l
        objects.add(referenceObject);      // ajoute  la fin
        shapes.put(referenceObject, new ShapeDescription(shape, color));
        redraw();
    }
 
    /**
     * Efface une forme donne de l'cran.
     * @param  referenceObject  L'objet forme  effacer 
     */
    public void erase(Object referenceObject)
    {
        objects.remove(referenceObject);   // au cas o il aurait dj t l
        shapes.remove(referenceObject);
        redraw();
    }

    /**
     * Dfinit la couleur de premier plan du fond.
     * @param  newColour   La nouvelle couleur du premier plan de Canvas 
     */
    public void setForegroundColor(String colorString)
    {
        if(colorString.equals("red"))
            graphic.setColor(Color.red);
        else if(colorString.equals("black"))
            graphic.setColor(Color.black);
        else if(colorString.equals("blue"))
            graphic.setColor(Color.blue);
        else if(colorString.equals("yellow"))
            graphic.setColor(Color.yellow);
        else if(colorString.equals("green"))
            graphic.setColor(Color.green);
        else if(colorString.equals("magenta"))
            graphic.setColor(Color.magenta);
        else if(colorString.equals("white"))
            graphic.setColor(Color.white);
        else
            graphic.setColor(Color.black);
    }

    /**
     * Attend un nombre spcifi de millisecondes avant de se terminer.
     * Ceci permet de spcifier facilement un petit dlai  
     * utiliser pour produire des animations.
     * @param  milliseconds  Le nombre de millisecondes
     */
    public void wait(int milliseconds)
    {
        try
        {
            Thread.sleep(milliseconds);
        } 
        catch (Exception e)
        {
            // ignore l'exception pour l'instant
        }
    }

    /**
     * Redessine toutes les formes actuellement dans le Canvas.
     */
    private void redraw()
    {
        erase();
        for(Iterator i=objects.iterator(); i.hasNext(); ) {
            ((ShapeDescription)shapes.get(i.next())).draw(graphic);
        }
        canvas.repaint();
    }
       
    /**
     * Efface la totalit du fond. (Ne redessine pas.)
     */
    private void erase()
    {
        Color original = graphic.getColor();
        graphic.setColor(backgroundColour);
        Dimension size = canvas.getSize();
        graphic.fill(new Rectangle(0, 0, size.width, size.height));
        graphic.setColor(original);
    }


    /************************************************************************
     * Partie interne de la classe CanvasPane - le composant rel du fond contenu dans
     * le cadre Canvas. Il s'agit essentiellement d'un JPanel avec des capacits supplmentaires
     * pour actualiser l'image qui y est dessine.
     */
    private class CanvasPane extends JPanel
    {
        public void paint(Graphics g)
        {
            g.drawImage(canvasImage, 0, 0, null);
        }
    }
    
    /************************************************************************
     * Partie interne de la classe CanvasPane - le composant rel du fond contenu dans
     * le cadre Canvas. Il s'agit essentiellement d'un JPanel avec des capacits supplmentaires
     * pour actualiser l'image qui y est dessine.
     */
    private class ShapeDescription
    {
        private Shape shape;
        private String colorString;

        public ShapeDescription(Shape shape, String color)
        {
            this.shape = shape;
            colorString = color;
        }

        public void draw(Graphics2D graphic)
        {
            setForegroundColor(colorString);
            graphic.fill(shape);
        }
    }

}
