import javax.swing.*;
import java.awt.*;
import java.util.List;
import java.util.*;

/**
 * Canvas est une classe qui permet de raliser un dessin graphique simple sur un fond.
 * Il s'agit d'une modification de la classe Canvas de but gnraliste, spcialement cre
 * l'exemple "shapes" de BlueJ. 
 *
 * @author: Bruce Quig
 * @author: Michael Kolling (mik)
 *
 * @version: 1.6 (shapes)
 */
public class Canvas
{
    // Note : l'implantation de cette classe (particulirement la gestion de
    // l'identit et des couleurs de la forme) est lgrement plus complexe que de besoin.
    // Ceci est fait  dessein pour que l'interface et les champs d'instance des
    // objets forme de ce projet restent simples, dans un but ducatif.

    private static Canvas canvasSingleton;

    /**
     * Mthode factory pour obtenir l'objet singleton de canvas.
     */
    public static Canvas getCanvas()
    {
        if(canvasSingleton == null) {
            canvasSingleton = new Canvas("Dmo Shapes de BlueJ", 300, 300, 
                                         Color.white);
        }
        canvasSingleton.setVisible(true);
        return canvasSingleton;
    }

    //  ----- partie instance -----

    private JFrame frame;
    private CanvasPane canvas;
    private Graphics2D graphic;
    private Color backgroundColour;
    private Image canvasImage;
    private List objects;
    private HashMap shapes;
    
    /**
     * Cre un Canvas.
     * @param title  Titre qui apparatra dans le cadre Canvas
     * @param width  Largeur souhaite pour le fond
     * @param height  Hauteur souhaite pour le fond
     * @param bgClour  Couleur souhaite pour l'arrire-plan du fond
     */
    private Canvas(String title, int width, int height, Color bgColour)
    {
        frame = new JFrame();
        canvas = new CanvasPane();
        frame.setContentPane(canvas);
        frame.setTitle(title);
        canvas.setPreferredSize(new Dimension(width, height));
        backgroundColour = bgColour;
        frame.pack();
        objects = new ArrayList();
        shapes = new HashMap();
    }

    /**
     * Dfinit la visibilit du fond et amne le fond  l'avant de l'cran
     * lorsqu'il est affich. Cette mthode peut galement permettre d'afficher
     * un fond dj visible  l'avant d'autres fentres.
     * @param visible  Valeur boolenne reprsentant la visibilit souhaite du
     * fond (true ou false) 
     */
    public void setVisible(boolean visible)
    {
        if(graphic == null) {
            // premire fois : instantie l'image hors cran et la remplit avec
            // la couleur d'arrire-plan
            Dimension size = canvas.getSize();
            canvasImage = canvas.createImage(size.width, size.height);
            graphic = (Graphics2D)canvasImage.getGraphics();
            graphic.setColor(backgroundColour);
            graphic.fillRect(0, 0, size.width, size.height);
            graphic.setColor(Color.black);
        }
        frame.setVisible(visible);
    }

    /**
     * Dessine une forme donne sur le fond.
     * @param  referenceObject  Un objet pour dfinir l'identit de cette forme
     * @param  color            La couleur de la forme
     * @param  shape            L'objet forme  dessiner sur le fond
     */
     // Note : ceci est une manire lgrement simple de conserver les objets
     // forme. Elle est soigneusement conue pour assurer que les interfaces des formes
     // visibles dans ce projet restent simples, dans un but ducatif.
    public void draw(Object referenceObject, String color, Shape shape)
    {
        objects.remove(referenceObject);   // au cas o il y serait dj
        objects.add(referenceObject);      // ajoute  la fin
        shapes.put(referenceObject, new ShapeDescription(shape, color));
        redraw();
    }
 
    /**
     * Efface une forme donne de l'cran.
     * @param  referenceObject  L'objet forme  effacer 
     */
    public void erase(Object referenceObject)
    {
        objects.remove(referenceObject);   // au cas o il serait dj
        shapes.remove(referenceObject);
        redraw();
    }

    /**
     * Dfinit la couleur de premier plan du Canvas.
     * @param  newColour   La nouvelle couleur de premier plan du Canvas 
     */
    public void setForegroundColor(String colorString)
    {
        if(colorString.equals("red"))
            graphic.setColor(Color.red);
        else if(colorString.equals("black"))
            graphic.setColor(Color.black);
        else if(colorString.equals("blue"))
            graphic.setColor(Color.blue);
        else if(colorString.equals("yellow"))
            graphic.setColor(Color.yellow);
        else if(colorString.equals("green"))
            graphic.setColor(Color.green);
        else if(colorString.equals("magenta"))
            graphic.setColor(Color.magenta);
        else if(colorString.equals("white"))
            graphic.setColor(Color.white);
        else
            graphic.setColor(Color.black);
    }

    /**
     * Attend un nombre spcif de millisecondes avant de terminer.
     * C'est une manire simple de spcifier un petit dlai qui peut tre
     * utilis pour produire des animations.
     * @param  milliseconds  Le nombre 
     */
    public void wait(int milliseconds)
    {
        try
        {
            Thread.sleep(milliseconds);
        } 
        catch (Exception e)
        {
            // ignore l'exception pour l'instant
        }
    }

    /**
     * Redessine toutes les formes actuellement dans Canvas.
     */
    private void redraw()
    {
        erase();
        for(Iterator i=objects.iterator(); i.hasNext(); ) {
            ((ShapeDescription)shapes.get(i.next())).draw(graphic);
        }
        canvas.repaint();
    }
       
    /**
     * Efface la totalit du fond (ne ractualise pas)
     */
    private void erase()
    {
        Color original = graphic.getColor();
        graphic.setColor(backgroundColour);
        Dimension size = canvas.getSize();
        graphic.fill(new Rectangle(0, 0, size.width, size.height));
        graphic.setColor(original);
    }


    /************************************************************************
     * Classe CanvasPane interne - Le vrai composant canvas contenu dans le
     * cadre Canvas. Il s'agit essentiellement d'un JPanel avec des capacits accrues pour
     * ractualiser l'image qui y est dessine.
     */
    private class CanvasPane extends JPanel
    {
        public void paint(Graphics g)
        {
            g.drawImage(canvasImage, 0, 0, null);
        }
    }
    
    /************************************************************************
     * Classe CanvasPane interne - Le vrai composant canvas contenu dans le
     * cadre Canvas. Il s'agit essentiellement d'un JPanel avec des capacits accrues pour
     * ractualiser l'image qui y est dessine.
     */
    private class ShapeDescription
    {
        private Shape shape;
        private String colorString;

        public ShapeDescription(Shape shape, String color)
        {
            this.shape = shape;
            colorString = color;
        }

        public void draw(Graphics2D graphic)
        {
            setForegroundColor(colorString);
            graphic.fill(shape);
        }
    }

}
