import java.util.ArrayList;
import java.util.Iterator;

/**
 * Un modle d'enchres simple.
 * L'enchre conserve une liste des lots de longueur arbitraire.
 * @author David J. Barnes et Michael Kolling.
 * @version 2001.06.08
 */
public class Auction
{
    // La liste des Lots de cette enchre.
    private ArrayList lots;
    // Le numro qui sera attribu au prochain lot entr
    // dans cette vente aux enchres.
    private int nextLotNumber;

    /**
     * Cre une nouvelle vente aux enchres.
     */
    public Auction()
    {
        lots = new ArrayList();
        nextLotNumber = 1;
    }

    /**
     * Entre un nouveau lot dans les enchres.
     * Les lots ne peuvent tre entrs que par un
     * objet Auction.
     * @param description Une description du lot.
     */
    public void enterLot(String description)
    {
        lots.add(new Lot(nextLotNumber, description));
        nextLotNumber++;
    }

    /**
     * Affiche la lite complte des numros et descriptions de lots dans
     * cette enchre. Inclut les dtails des offres les mieux-disantes.
     */
    public void showLots()
    {
        Iterator it = lots.iterator();
        while(it.hasNext()) {
            Lot lot = (Lot) it.next();
            System.out.println(lot.getNumber() + ": " +
                               lot.getDescription());
            // Inclut les dtails de l'offre la mieux-disante.
            Bid highestBid = lot.getHighestBid();
            if(highestBid != null) {
                System.out.println("    Offre : " + 
                                   highestBid.getValue());
            }
            else {
                System.out.println("    (Pas d'offre)");
            }
        }
    }

    /**
     * Renvoie le lot avec le numro donn. Renvoie null
     * s'il n'existe pas de lot avec ce numro.
     * @param number Le numro du lot  renvoyer.
     */
    public Lot getLot(int number)
    {
        if((number >= 1) && (number < nextLotNumber)) {
            // Le numro semble raisonnable.
            Lot selectedLot = (Lot) lots.get(number-1);
            // Inclut une vrification de confiance pour tre srs que
            // nous avons le bon lot.
            if(selectedLot.getNumber() != number) {
                System.out.println("Erreur interne : " +
                                   "Lot renvoy erron. " +
                                   "Numro : " + number);
            }
            return selectedLot;
        }
        else {
            System.out.println("Le numro de lot : " + number +
                               " n'existe pas.");
            return null;
        }
    }
}
