/**
 * Une classe permettant de modliser un lment (ou un ensemble d'lments) dans une
 * enchre : un lot.
 * @author David J. Barnes et Michael Kolling.
 * @version 2001.06.08
 */
public class Lot
{
    // Un numro d'identification unique.
    private final int number;
    // Une description du lot.
    private String description;
    // L'offre actuelle la mieux-disante pour ce lot.
    private Bid highestBid = null;

    /**
     * Construit un Lot, en dfinissant son numro et sa description.
     * @param number Le numro du lot.
     * @param description Une description de ce lot.
     */
    public Lot(int number, String description)
    {
        this.number = number;
        this.description = description;
    }

    /**
     * Tentative d'enchrir sur ce lot. Une offre gagnante
     * doit avoir une valeur suprieure  toute offre prcdente.
     * @param bidder L'enchrisseur.
     * @param value La valeur de l'offre.
     */
    public void bidFor(Person bidder, long value)
    {
        // Nous admettons que le lot est authentique. Rien n'empchera la
        // prsence d'une offre pour un faux lot, mais cela
        // n'apparatra pas dans la liste des enchres.
        if((highestBid == null) ||
               (highestBid.getValue() < value)) {
            // Cette offre est la mieux-disante jusqu' maintenant.
            setHighestBid(new Bid(bidder, value));
        }
        else {
            System.out.println("Numro du lot : " + getNumber() +
                               " (" + getDescription() + ")" +
                               " a dj une offre de : " +
                               highestBid.getValue());
        }
    }

    /**
     * @return Le numro du lot.
     */
    public int getNumber()
    {
        return number;
    }

    /**
     * @return La description du lot.
     */
    public String getDescription()
    {
        return description;
    }

    /**
     * @return L'offre la mieux-disante pour ce lot. Pourrait tre null s'il
     *         n'y a pas eu d'offres.
     */
    public Bid getHighestBid()
    {
        return highestBid;
    }

    /**
     * @param highestBid La nouvelle offre la mieux-disante.
     */
    private void setHighestBid(Bid highestBid)
    {
        this.highestBid = highestBid;
    }
}
