/**
 * Mmorise les donnes d'une seule ligne d'un
 * fichier journal de serveur Web.
 * Les champs individuels sont mis  disposition via
 * des accesseurs comme getHour() et getMinute().
 * 
 * @author David J. Barnes et Michael Kolling.
 * @version 2002.01.01
 */
public class LogEntry implements Comparable
{
    // L'endroit o les valeurs de donnes extraites d'une seule
    // ligne de journal sont stockes.
    private int[] dataValues;
    // L'index auquel, dans dataValues, les diffrents champs d'une
    // ligne de journal sont stocks.
    private final int YEAR = 0, MONTH = 1, DAY = 2,
                      HOUR = 3, MINUTE = 4;
                      
    /**
     * Dcompose une ligne de journal afin que chaque champ
     * soit disponible.
     * @param logline Une seule ligne de journal.
     */
    public LogEntry(String logline)
    {
        // Le tableau pour stocker les donnes d'une seule ligne.
        dataValues = new int[5];
        // Dcompose la ligne de journal.
        LoglineTokenizer tokenizer = new LoglineTokenizer();
        tokenizer.tokenize(logline,dataValues);
    }
    
    /**
     * @return Le champ hour de la ligne de journal.
     */
    public int getHour()
    {
        return dataValues[HOUR];
    }

    /**
     * @return Le champ minute de la ligne de journal.
     */
    public int getMinute()
    {
        return dataValues[MINUTE];
    }
    
    /**
     * Cre une reprsentation en chane des donnes.
     * Ceci ne doit pas forcment tre identique au
     * texte de la ligne de journal d'origine.
     *
     * @return Une chane reprsentant les donnes de cette entre.
     */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        for(int i = 0; i < dataValues.length; i++) {
            int value = dataValues[i];
            // Place un premier zro devant les nombres  un chiffre.
            if(value < 10) {
                buffer.append('0');
            }
            buffer.append(value);
            buffer.append(' ');
        }
        // Efface tout espace inutile.
        return buffer.toString().trim();
    }
    
    /**
     * Compare la combinaison date/heure de cette entre de journal
     * avec une autre.
     * @param other L'autre entre pour la comparaison.
     * @return Une valeur ngative si cette entre vient avant l'autre.
     *         Une valeur positive si cette entre vient aprs l'autre.
     *         Zro si les entres sont identiques.
     */
    public int compareTo(Object other)
    {
        if(other == this) {
            // Ce sont les mmes objets.
            return 0;
        }
        else {
            LogEntry otherEntry = (LogEntry) other;
            // Compare les champs concordants.
            for(int i = 0; i < dataValues.length; i++) {
                int difference = dataValues[i] - otherEntry.dataValues[i];
                if(difference != 0) {
                    return difference;
                }
            }
            // Les valeurs ne sont pas diffrentes, les deux entres reprsentent
            // des heures identiques.
            return 0;
        }
    }
}
