import java.awt.*;
import java.awt.geom.*;

/**
 * Classe BouncingBall : une balle graphique qui observe l'effet de la gravit. La balle
 * peut bouger. Les dtails des mouvements sont dtermins par la balle elle-mme. Elle
 * tombe, en acclrant  cause de la gravit et rebondit
 * vers le haut lorsqu'elle frappe le sol.
 *
 * Ce mouvement peut tre initi par des appels rpt  la mthode move.
 * 
 * @author Bruce Quig
 * @author Michael Kolling (mik)
 * @author David J. Barnes
 *
 * @version 1.1  (23-Jan-2002)
 */

public class BouncingBall
{
    private static final int gravity = 3;  // effet de gravit

    private int ballDegradation = 2;
    private Ellipse2D.Double circle;
    private Color color;
    private int diameter;
    private int xPosition;
    private int yPosition;
    private final int groundPosition;      // position y du sol
    private Canvas canvas;
    private int ySpeed = 1;                // vitesse initiale vers le bas

    /**
     * Constructeur d'objets de la classe BouncingBall
     *
     * @param xPos  les coordonnes horizontales de la balle
     * @param yPos  les coordonnes verticales de la balle
     * @param ballDiameter  le diamtre (en pixels) de la balle
     * @param ballColor  la couleur de la balle
     * @param groundPos  la position du sol (o la balle rebondira)
     * @param drawingCanvas  le fond sur lequel dessiner la balle
     */
    public BouncingBall(int xPos, int yPos, int ballDiameter, Color ballColor,
                        int groundPos, Canvas drawingCanvas)
    {
        xPosition = xPos;
        yPosition = yPos;
        color = ballColor;
        diameter = ballDiameter;
        groundPosition = groundPos;
        canvas = drawingCanvas;
    }

    /**
     * Dessine cette balle  sa position actuelle sur le fond.
     **/
    public void draw()
    {
        canvas.setForegroundColor(color);
        canvas.fillCircle(xPosition, yPosition, diameter);
    }

    /**
     * Efface cette balle de sa position actuelle.
     **/
    public void erase()
    {
        canvas.eraseCircle(xPosition, yPosition, diameter);
    }    

    /**
     * Dplace cette balle selon sa position et sa vitesse et la redessine.
     **/
    public void move()
    {
        // supprime du fond  la position en cours
        erase();
            
        // calcule la nouvelle position
        ySpeed += gravity;
        yPosition += ySpeed;
        xPosition +=2;

        // vrifie si elle a frapp le sol
        if(yPosition >= (groundPosition - diameter) && ySpeed > 0) {
            yPosition = (int)(groundPosition - diameter);
            ySpeed = -ySpeed + ballDegradation; 
        }

        // redessine l'objet  la nouvelle position
        draw();
    }    

    /**
     * renvoie la position horizontale de cette balle
     */
    public int getXPosition()
    {
        return xPosition;
    }

    /**
     * renvoie la position verticale de cette balle
     */
    public int getYPosition()
    {
        return yPosition;
    }

}
