import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.Set;
import java.util.HashSet;

/**
 * La classe InputReader lit le texte saisi dans le terminal de texte standard. Le texte
 * tap par un utilisateur est alors dcoup en mots et un ensemble de mots est fourni.
 * 
 * @version    1.0  (21.Jan.2002)
 * @author     Michael Kolling et David J. Barnes
 */
public class InputReader
{
    private BufferedReader reader;

    /**
     * Cre un nouvel InputReader qui lit du texte  partir du terminal.
     */
    public InputReader()
    {
        reader = new BufferedReader(new InputStreamReader(System.in));;
    }

    /**
     * Lit une ligne de texte  partir de la saisie standard (le terminal) et
     * la renvoie sous la forme d'un ensemble de mots.
     *
     * @return  Un ensemble de String, o chacun est l'un des mots
     * tap par l'utilisateur
     */
    public HashSet getInput() 
    {
        System.out.print("> ");                // affiche l'invite
        String inputLine = readInputLine().trim().toLowerCase();

        StringTokenizer tokenizer = new StringTokenizer(inputLine);
        HashSet words = new HashSet();

        while(tokenizer.hasMoreTokens())
            words.add(tokenizer.nextToken());      // ajoute chaque mot  l'ensemble

        return words;
    }

    /**
     * Lit une ligne de saisie et la renvoie sous forme de String. 
     *
     * @return  Un String reprsentant la saisie ou un String vide en cas d'erreur.
     */
    private String readInputLine()
    {
        String line = "";

        try {
            line = reader.readLine();
        }
        catch(java.io.IOException exc) {
            System.out.println ("Erreur au cours de la lecture : " + exc.getMessage());
        }
        return line;
    }
}
