/**
 * La partie principale du calculateur ralisant
 * la logique arithmtique des calculs.
 * @author Hacker T. Largebrain (version 0.1)
 * @version 0.2
 */
public class CalcEngine
{
    // La valeur de l'affichage.
    private int displayValue;
    // L'oprateur saisi prcdemment (+ ou -).
    private char previousOperator;
    // L'oprande gauche de previousOperator.
    private int leftOperand;

    /**
     * Cration d'une instance de CalcEngine.
     */
    public CalcEngine()
    {
        displayValue = 0;
        previousOperator = ' ';
        leftOperand = 0;
    }

    /**
     * @return La valeur actuellement affiche
     * par le calculateur.
     */
    public int getDisplayValue()
    {
        return displayValue;
    }

    /**
     * Un bouton associ  un chiffre est appuy.
     */
    public void numberPressed(int number)
    {
        System.out.println("numberPressed appele avec : " +
                           number);
        displayValue = displayValue * 10 + number;
        reportState("fin de numberPressed");
    }

    /**
     * Appele quand l'oprateur plus est appuy.
     */
    public void plus()
    {
        System.out.println("plus appele");
        applyPreviousOperator();
        previousOperator = '+';
        displayValue = 0;
        reportState("fin de plus");
    }

    /**
     * Appele quand loprateur moins est appuy.
     */
    public void minus()
    {
        applyPreviousOperator();
        previousOperator = '-';
        displayValue = 0;
    }
    
    /**
     * Appele pour terminer un calcul.
     */
    public void equals()
    {
        System.out.println("equals appele");
        if(previousOperator == '+') {
            displayValue = leftOperand + displayValue;
        }
        else {
            displayValue = leftOperand - displayValue;
        }
        leftOperand = 0;
        reportState("fin de equals");
    }

    /**
     * Appele pour rinitialiser la calculatrice.
     */
    public void clear()
    {
        System.out.println("clear appele");
        displayValue = 0;
        reportState("fin de clear");
    }

    /**
     * Renvoie le titre du moteur de calcul.
     */
    public String getTitle()
    {
        return "Super Calculator";
    }

    /**
     * Renvoie l'auteur de ce moteur.
     */
    public String getAuthor()
    {
        return "Hacker T. Largebrain";
    }

    /**
     * Renvoie le numro de version de ce moteur.
     */
    public String getVersion()
    {
        return "version 0.2";
    }

    /**
     * Affiche les valeurs des champs de cet objet.
     * @param where o l'tat a t rencontr.
     */
    public void reportState(String where)
    {
        System.out.println("displayValue : " + displayValue +
                           " leftOperand : " + leftOperand +
                           " previousOperator : " +
                           previousOperator + "  " + where);
    }
    
    /**
     * Un bouton associ  un oprateur a t appuy.
     * Appliquer immdiatement l'oprateur prcdent pour 
     * calculer un rsultat intermdiaire. Ce rsultat sera
     * l'oprande gauche du nouvel oprateur.
     */
    private void applyPreviousOperator()
    {
        System.out.println("applyPreviousOperator appele");
        if(previousOperator == '+') {
            leftOperand += displayValue;
        }
        else if(previousOperator == '-') {
            leftOperand -= displayValue;
        }
        else {
            // Il n'y a avait pas d'oprateur prcdent.
            leftOperand = displayValue;
        }
        reportState("fin de applyPreviousOperator");
    }
}
