/**
 * La partie principale de la calculatrice ralisant les calculs.
 * 
 * @author  David J. Barnes and Michael Kolling 
 * @version 1.0
 */
public class CalcEngine
{
    // L'tat de la calculatrice est reprsent par trois champs:
    //     buildingDisplayValue, haveLeftOperand, and lastOperator.
    
    // Sommes-nous dj en cours d'affichage d'une valeur ou est ce 
    // que le prochain chiffre est le premier d'une nouvelle valeur ?
    private boolean buildingDisplayValue;
    // Est ce qu'une oprande gauche a dj t saisie (ou calcule) ?
    private boolean haveLeftOperand;
    // L'oprateur saisi le plus rcent.
    private char lastOperator;

    // La valeur courante  afficher.
    private int displayValue;
    // La valeur d'une oprande gauche dj existante.
    private int leftOperand;

    /**
     * Cration d'une instance de CalcEngine.
     */
    public CalcEngine()
    {
        clear();
    }

    /**
     * Renvoie la valeur qui devrait tre 
     * affiche actuellement par l'cran de la calculatrice.
     */
    public int getDisplayValue()
    {
        return displayValue;
    }

    /**
     * Un bouton associ  un chiffre est appuy.
     * Soit nous crons une nouvelle oprande, soit nous
     * considrons cette valeur comme le chiffre le moins significatif
     * d'une oprande dj existante.
     */
    public void numberPressed(int number)
    {
        if(buildingDisplayValue) {
            // Chiffre le moins significatif
            displayValue = displayValue*10 + number;
        }
        else {
            // Cration d'une nouvelle valeur.
            displayValue = number;
            buildingDisplayValue = true;
        }
    }

    /**
     * Le bouton 'plus' est appuy. 
     */
    public void plus()
    {
        applyOperator('+');
    }

    /**
     * Le bouton 'moins' est appuy. 
     */
    public void minus()
    {
        applyOperator('-');
    }
    
    /**
     * Le bouton '=' est appuy.
     */
    public void equals()
    {
        // Cela devrait terminer la construction de la deuxime oprande,
        // donc nous nous assurons que nous avons rellement une oprande
        // gauche, un oprateur et une oprande droite.
        if(haveLeftOperand &&
                lastOperator != '?' &&
                buildingDisplayValue) {
            calculateResult();
            lastOperator = '?';
            buildingDisplayValue = false;
        }
        else {
            keySequenceError();
        }
    }

    /**
     * Le bouton 'C' (rinitialiser) est appuy.
     * Rinitialise l'ensemble des variables comme au dpart.
     */
    public void clear()
    {
        lastOperator = '?';
        haveLeftOperand = false;
        buildingDisplayValue = false;
        displayValue = 0;
    }

    /**
     * Renvoie le titre du moteur de calcul.
     */
    public String getTitle()
    {
        return "Java Calculator";
    }

    /**
     * Renvoie l'auteur de ce moteur.
     */
    public String getAuthor()
    {
        return "David J. Barnes and Michael Kolling";
    }

    /**
     * Renvoie le numro de version de ce moteur.
     */
    public String getVersion()
    {
       return "Version 1.0";
    }

    /**
     * Combine leftOperand, lastOperator et la valeur actuellement affiche.
     * Le rsultat devient  la fois la valeur de leftOperand et
     * la nouvelle valeur affiche.
     */
    private void calculateResult()
    {
        switch(lastOperator) {
            case '+':
                displayValue = leftOperand + displayValue;
                haveLeftOperand = true;
                leftOperand = displayValue;
                break;
            case '-':
                displayValue = leftOperand - displayValue;
                haveLeftOperand = true;
                leftOperand = displayValue;
                break;
            default:
                keySequenceError();
                break;
        }
    }
    
    /**
     * Applique l'oprateur transmis.
     */
    private void applyOperator(char operator)
    {
        // Si nous ne sommes pas en train de construire un nouvel oprateur,
        // c'est une erreur  moins que nous venions juste de calculer un rsultat
        // en utilisant '='.
        if(!buildingDisplayValue &&
                    !(haveLeftOperand && lastOperator == '?')) {
            keySequenceError();
            return;
        }

        if(lastOperator != '?') {
            // Nous appliquons en premier l'oprateur prcdent.
            calculateResult();
        }
        else {
            // La valeur de displayValue devient maintenant l'oprande gauche du 
            // nouvel operateur.
            haveLeftOperand = true;
            leftOperand = displayValue;
        }
        lastOperator = operator;
        buildingDisplayValue = false;
    }

    /**
     * Signale une erreur dans la squence des touches appuyes.
     */
    private void keySequenceError()
    {
        System.out.println("Une erreur dans la squence des touches est apparues.");
        // Rinitialisation du tout.
        clear();
    }
}
