/**
 * Reprsente une semaine compose de journes.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2001.09.14
 */
public class Week
{
    // Reprsente une semaine comme un ensemble de journes du lundi au vendredi.
    public static final int BOOKABLE_DAYS_PER_WEEK = 5;
    
    // Un numro de semaine pour une anne prcise (0-52).
    private final int weekNumber;
    private final Day[] days;

    /**
     * Constructeur pour les objets de la classe Week.
     * @param weekNumber Le numro de la semaine (0-52).
     */
    public Week(int weekNumber)
    {
        this.weekNumber = weekNumber;
        days = new Day[BOOKABLE_DAYS_PER_WEEK];
        int dayInYear = weekNumber * 7 + 1;
        for(int day = 0; day < BOOKABLE_DAYS_PER_WEEK; day++) {
            days[day] = new Day(dayInYear);
            dayInYear++;
        }
    }

    /**
     * Affiche une liste de rendez-vous pour cette semaine sur 
     * la sortie standard.
     */
    public void showAppointments()
    {
        System.out.println("*** Semaine " + weekNumber + " ***");
        for(int i = 0; i < BOOKABLE_DAYS_PER_WEEK; i++) {
            days[i].showAppointments();
        }
    }

    /**
     * @param dayInWeek Quel jour (1..BOOKABLE_DAYS_PER_WEEK).
     * @return Le jour reprsentant ce numro ou null si dayInWeek est 
     * en dehors de l'intervalle.
     */
    public Day getDay(int dayInWeek)
    {
        if(dayInWeek >= 1 && dayInWeek <= BOOKABLE_DAYS_PER_WEEK) {
            return days[dayInWeek - 1];
        }
        else {
            return null;
        }
    }

    /**
     * @return Le numro de la semaine (0-52).
     */
    public int getWeekNumber()
    {
        return weekNumber;
    }
}
