/**
 * Ralise les tests de la classe Day  qui implique la 
 * prise de rendez-vous d'une heure. Automatise ces tests
 * autant que possible.
 * 
 * @author David J. Barnes et Michael Kolling. 
 * @version 2001.09.18
 */
public class AutomatedOneHourTests
{
    // L'objet Day qui sera test.
    private Day day;

    /**
     * Constructeur des objets de la classe AutomatedOneHourTests
     */
    public AutomatedOneHourTests()
    {
        // Cration d'un objet Day qui peut tre utilis pour le test.
        // Chaque mthode peut choisir de crer ses propres instances.
        day = new Day(1);
    }

    /**
     * Excute chaque test et vrifie le rsultat. Si un rsultat indique
     * un problme possible, affiche un message d'erreur
     * @return Les tests se sont-ils bien drouls ?
     */
    public boolean runAllTests()
    {   
        boolean allPassed = true;
        System.out.println("runAllTests dmarre...");
        if(!makeThreeAppointments()) {
            System.out.println("makeThreeAppointments() a chou.");
            allPassed = false;
        }
        if(!testDoubleBooking()) {
            System.out.println("testDoubleBooking() a chou.");
            allPassed = false;
        }
        if(!fillTheDay()) {
            System.out.println("fillTheDay() a chou.");
            allPassed = false;
        }
        System.out.println("runAllTests termine");
        return allPassed;
    }

    /**
     * Test de la fonctionnalit lmentaire en rservant 
     * en dbut, milieu et fin de journe.
     * @return true si le test est un succs, false sinon.
     */
    public boolean makeThreeAppointments()
    {
        // Cration dun nouvel objet Day.
        day = new Day(1);
        // Cration de trois rendez-vous dune heure.
        Appointment first = new Appointment("Java lecture", 1);
        Appointment second = new Appointment("Java class", 1);
        Appointment third = new Appointment("Meet John", 1);
        
        // Suppose que le test est un succs 
        //  moins que nous dcouvrions une erreur.
        boolean passed = true;
        // Prendre chaque rendez-vous  des 
        // heures diffrentes.
        passed &= day.makeAppointment(9, first);
        passed &= day.makeAppointment(13, second);
        passed &= day.makeAppointment(17, third);
        
        // Vrifie que chaque rendez-vous a t rserv 
        //  la bonne heure.
        passed &= day.getAppointment(9) == first;
        passed &= day.getAppointment(13) == second;
        passed &= day.getAppointment(17) == third;
        return passed;
    }
    
    /**
     * Vrifie que la double rservation nest pas permise.
     * @return true si le test est un succs, false sinon.
     */
    public boolean testDoubleBooking()
    {
        // Suppose que le test est un succs 
        //  moins que nous dcouvrions une erreur.
        boolean passed = true;
        // Initialisation de la journe avec trois 
        // rendez-vous lgitimes.
        passed = makeThreeAppointments();
        Appointment badAppointment = new Appointment("Error", 1);
        
        // L'heure que nous allons essay de rserver deux fois.
        int time = 9;
        // Ralise une copie du rendez-vous  cette heure,
        // pour s'assurer qu'il n'est pas modifi par le prochain test.
        Appointment first = day.getAppointment(time);
        // Notons que nous voulons que ce test choue.
        passed &= !day.makeAppointment(time, badAppointment);
        // Nous vrifions que le premier est toujours la o il devrait tre.
        passed &= day.getAppointment(time) == first;
        return passed;
    }

    /**
     * Vrifie la fonctionnalit lmentaire en 
     * remplissant compltement une journe de rendez-vous.
     * @return true si le test est un succs, false sinon.
     */
    public boolean fillTheDay()
    {
        // Cration dun nouvel objet Day.
        day = new Day(1);
        // Nous nous assurons que tous les rendez-vous peuvent tre pris.
        boolean passed = true;
        for(int time = Day.START_OF_DAY;
                    time <= Day.FINAL_APPOINTMENT_TIME;
                        time++) {
            passed &= day.makeAppointment(time,
                                new Appointment("Test " + time, 1));
        }
        // Nous vrifions que toutes les heures des rendez-vous sont occupes.
        for(int time = Day.START_OF_DAY;
                    time <= Day.FINAL_APPOINTMENT_TIME;
                        time++) {
            passed &= day.getAppointment(time) != null;
        }
        return passed;
    }
}
