/**
 * Maintient les rendez-vous pour une journe dans un agenda.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2001.09.14
 */
public class Day
{
    // La premire et la dernire heure rservable dans une journe.
    public static final int START_OF_DAY = 9;
    public static final int FINAL_APPOINTMENT_TIME = 17;
    // Le nombre dheures qui peuvent tre rserves dans une journe.
    public static final int MAX_APPOINTMENTS_PER_DAY =
                                FINAL_APPOINTMENT_TIME -
                                START_OF_DAY + 1;
    
    // Un numro de journe dans une anne particulire. (1-366)
    private int dayNumber;
    // La liste courante des rendez-vous pour cette journe.
    private final Appointment[] appointments;

    /**
     * Constructeur pour les objets de la classe Day.
     * @param dayNumber Le numro de cette journe dans lanne (1-366).
     */
    public Day(int dayNumber)
    {
        this.dayNumber = dayNumber;
        appointments = new Appointment[MAX_APPOINTMENTS_PER_DAY];
    }

    /**
     * Recherche dune place libre pour un rendez-vous.
     * @param appointment Le rendez-vous  traiter.
     * @return La premire heure dans la journe disponible
     *         pour le rendez-vous. Renvoie -1 sil ny a pas
     *         de place disponible.
     */ 
    public int findSpace(Appointment appointment)
    {
        int duration = appointment.getDuration();
        for(int slot = 0; slot < MAX_APPOINTMENTS_PER_DAY; slot++) {
            if(appointments[slot] == null) {
                final int time = START_OF_DAY + slot;
                // Point de dpart potentiel.
                if(duration == 1) {
                    // seulement un crneau est ncessaire.
                    return time;
                }
                else {
                    // Combien de crneaux supplmentaires sont ncessaires ?
                    int further_slots_required = duration - 1;
                    for(int nextSlot = slot + 1;
                                further_slots_required > 0 &&
                                appointments[nextSlot] == null;
                                    nextSlot++) {
                        further_slots_required--;
                    }
                    if(further_slots_required == 0) {
                        // Un espace suffisamment grand a t trouv.
                        return time;
                    }
                }
            }
        }
        // Il ny a pas assez de crneaux disponibles.
        return -1;
    }

    /**
     * Prendre un rendez-vous.
     * @param time Lheure de dbut du rendez-vous.
     * @param appointment Le rendez-vous  prendre.
     * @return true si le rendez-vous a t pris, false sinon.
     */
    public boolean makeAppointment(int time,
                                   Appointment appointment)
    {
        if(time >= START_OF_DAY && time <= FINAL_APPOINTMENT_TIME) {
            int startTime = time-START_OF_DAY;
            if(appointments[startTime] == null) {
                int duration = appointment.getDuration();
                // Rempli tous les crneaux pour 
                // la dure totale du rendez-vous.
                for(int i = 0; i < duration; i++) {
                    appointments[startTime + i] = appointment;
                }
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }

    /**
     * Imprime une liste des rendez-vous de la journe 
     * sur la sortie standard.
     */
    public void showAppointments()
    {
        System.out.println("=== Jour " + dayNumber + " ===");
        for(int i = 0; i < MAX_APPOINTMENTS_PER_DAY; i++) {
            System.out.print((START_OF_DAY + i) + ": ");
            if(appointments[i] != null) {
                System.out.println(
                    appointments[i].getDescription());
            }
            else {
                System.out.println();
            }
        }
    }

    /**
     * @return Le numro de cette journe dans lanne (1 - 366).
     */
    public int getDayNumber()
    {
        return dayNumber;
    }
}
