/**
 * Cette classe est la classe principale de l'application "le monde de Zuul".
 * "le monde de Zuul" est un jeu d'aventures trs simple en mode texte.
 * Les utilisateurs peuvent se promener dans le dcor. C'est tout. 
 * Il devrait vraiment tre tendu pour le rendre plus intressant!
 * 
 * Pour jouer  ce jeu,  crez une instance de cette classe 
 * et appelez la mthode "play".
 * 
 * Cette classe principale cre et initialise tous les autres: 
 * elle cre toutes les pices, cre l'analyseur syntaxique et dmarre le jeu.  
 * Elle value aussi et excute les commandes renvoyes par l'analyseur syntaxique. 
 * 
 * @author  Michael Kolling and David J. Barnes
 * @version 1.0 (February 2002)
 */

class Game 
{
    private Parser parser;
    private Room currentRoom;
        
    /**
     * Cration du jeu et initialisation de sa carte interne.
     */
    public Game() 
    {
        createRooms();
        parser = new Parser();
    }

    /**
     * Cration des pices et des liens entre les sorties des pices
     */ 
    private void createRooms()
    {
        Room outside, theatre, pub, lab, office;
      
        // cration des pices
        outside = new Room(" l'extrieur de l'entre principale de l'Universit");
        theatre = new Room("dans un amphithtre");
        pub = new Room(" la caftria");
        lab = new Room("dans la salle informatique");
        office = new Room("au bureau des techniciens");
        
        // initialisation des sorties des pices
        outside.setExits(null, theatre, lab, pub);
        theatre.setExits(null, null, null, outside);
        pub.setExits(null, outside, null, null);
        lab.setExits(outside, office, null, null);
        office.setExits(null, null, null, lab);

        currentRoom = outside;  // dbut de la partie ailleurs
    }

    /**
     *  Fonction principale de jeu.  Boucle jusqu' la fin du jeu.
     */
    public void play() 
    {            
        printWelcome();

        // Entre dans la boucle principale des commandes.  Ici, nous
        // rptons la lecture et l'excution des commandes 
        // jusqu' la fin du jeu.
                
        boolean finished = false;
        while (! finished) {
            Command command = parser.getCommand();
            finished = processCommand(command);
        }
        System.out.println("Merci d'avoir jou.  Au revoir.");
    }

    /**
     * Affichage du message d'accueil au joueur.
     */
    private void printWelcome()
    {
        System.out.println();
        System.out.println("Bienvenue au jeu de Zuul!");
        System.out.println("Zuul est un nouveau jeu d'aventure terriblement ennyeux.");
        System.out.println("Tapez 'aide' si vous avez besoin d'aide.");
        System.out.println();
        System.out.println("Vous tes dans " + currentRoom.getDescription());
        System.out.print("Les sorties: ");
        if(currentRoom.northExit != null)
            System.out.print("nord ");
        if(currentRoom.eastExit != null)
            System.out.print("est ");
        if(currentRoom.southExit != null)
            System.out.print("sud ");
        if(currentRoom.westExit != null)
            System.out.print("ouest ");
        System.out.println();
    }

    /**
     * Etant donne une commande, traiter (ie excuter) la commande.
     * Si cette commande termine le jeu, true est renvoye, sinon la
     * la valeur false est renvoye.
     */

    private boolean processCommand(Command command) 
    {
        boolean wantToQuit = false;

        if(command.isUnknown()) {
            System.out.println("Je ne comprends pas cette commande...");
            return false;
        }

        String commandWord = command.getCommandWord();
        if (commandWord.equals("aide"))
            printHelp();
        else if (commandWord.equals("aller"))
            goRoom(command);
        else if (commandWord.equals("quitter"))
            wantToQuit = quit(command);

        return wantToQuit;
    }

    // implantation des commandes:

    /**
     * Affichage de l'aide.
     * Nous affichons ici quelque message stupide, mystrieux et 
     * une liste des ordres.
     */
    private void printHelp() 
    {
	System.out.println("Vous tes perdu. Vous tes seul. Vous flnez");
	System.out.println(" travers l'universit.");
	System.out.println();
	System.out.println("Les commandes sont :");
	System.out.println("   aller quitter aide");
    }

    /** 
     * Essai de dplacement dans une direction. 
     * Si une sortie existe, nous entrons dans la nouvelle pice,           
     * sinon afficher un message d'erreur
     */
    private void goRoom(Command command) 
    {
        if(!command.hasSecondWord()) {
            // en cas d'absence du deuxime mot, 
	    // nous ne savons pas o aller
            System.out.println("Aller o?");
            return;
        }

        String direction = command.getSecondWord();

        // nous essayons de quitter la pice courante.
        Room nextRoom = null;
        if(direction.equals("nord"))
            nextRoom = currentRoom.northExit;
        if(direction.equals("est"))
            nextRoom = currentRoom.eastExit;
        if(direction.equals("sud"))
            nextRoom = currentRoom.southExit;
        if(direction.equals("ouest"))
            nextRoom = currentRoom.westExit;

        if (nextRoom == null)
            System.out.println("Il n'y a pas de porte!");
        else {
            currentRoom = nextRoom;
            System.out.println("Vous tes " + currentRoom.getDescription());
            System.out.print("Les sorties: ");
            if(currentRoom.northExit != null)
                System.out.print("nord ");
            if(currentRoom.eastExit != null)
                System.out.print("est ");
            if(currentRoom.southExit != null)
                System.out.print("sud ");
            if(currentRoom.westExit != null)
                System.out.print("ouest ");
            System.out.println();
        }
    }
    
    /** 
     * "Quitter" viens d'tre saisi. Vrification du deuxime mot de la commande
     * pour s'assurer que nous voulons relement quitter le jeu. Renvoie la valeur 
     * true dans ce cas, false sinon.
     */
    private boolean quit(Command command) 
    {
        if(command.hasSecondWord()) {
            System.out.println("Quitter quoi?");
            return false;
        }
        else
            return true;  // informe que nous voulons quitter le jeu
    }
    
    public static void main(String[] args) {
    	Game g = new Game();
    	g.createRooms();
    	g.printWelcome();
    	g.play();
    }
}
