/**
 * Cette classe est la classe principale de l'application "le monde de Zuul".
 * "le monde de Zuul" est un jeu d'aventures trs simple en mode texte.
 *
 * Cette classe stocke les informations concernant une commande de l'utilisateur.
 * Une commande consiste actuellement en deux chanes: un mot d'instruction et 
 * un deuxime mot (par exemple, si la commande tait "prendre carte", 
 * alors les deux chanes sont videmment "prendre" et "carte").
 * 
 * Voici l'utilisation qui en est faite: les commandes sont dj vrifies pour 
 * tre des mots d'instruction valides. Si l'utilisateur avait entr une commande errone 
 * (un mot qui n'est pas connu) alors le mot d'instruction est <null>.
 *
 * Si la commande n'est constitue que d'un seul mot, le second est alors <null>.
 * 
 * @author  Michael Kolling and David J. Barnes
 * @version 1.0 (February 2002)
 */

class Command
{
    private String commandWord;
    private String secondWord;

    /**
     * Cration d'un objet de type commande. Le premier et le second mot doivent tre fourni, 
     * mais l'un ou l'autre (ou les deux) peuvent tre null. Le mot d'instruction doit tre null
     * pour indiquer que cette commande n'est pas reconnu par le jeu.
     */
    public Command(String firstWord, String secondWord)
    {
        commandWord = firstWord;
        this.secondWord = secondWord;
    }

    /**
     * Renvoie le mot d'instruction (le premier mot) de la commande.
     * Si la commande n'est pas comprise, le rsultat est null.
     */
    public String getCommandWord()
    {
        return commandWord;
    }

    /**
     * Renvoie le deuxime mot de cette commande. Renvoie null s'il n'y
     * avait pas de deuxime mot.
     */
    public String getSecondWord()
    {
        return secondWord;
    }

    /**
     * Renvoie true si la commande n'a pas t comprise.
     */
    public boolean isUnknown()
    {
        return (commandWord == null);
    }

    /**
     * Renvoie true si la commande est compose de deux mots.
     */
    public boolean hasSecondWord()
    {
        return (secondWord != null);
    }
}

