import java.util.Set;
import java.util.HashMap;
import java.util.Iterator;

/*
 * Class Room - une pice dans un jeu d'aventures.
 *
 * Cette classe est la classe principale de l'application "le monde de Zuul".
 * "le monde de Zuul" est un jeu d'aventures trs simple en mode texte.
 *
 * Une pice ("Room") reprsente un lieu dans le scnario du jeu.
 * Elle est relie  d'autres pices  l'aide de sorties. Les sorties
 * sont nommes nord, est, sud, ouest. Pour chaque direction, la pice stocke une rfrence
 * vers la pice voisine ou null s'il n'existe pas de sortie dans cette direction.
 * 
 * @author  Michael Kolling and David J. Barnes
 * @version 1.0 (February 2002)
 */

class Room 
{
    private String description;
    private HashMap exits;        // sauvegarde des sorties de la pice.

    /**
     * Cre une pice dcrite par la chane 'description'.
     * Au dpart, il n'existe aucune sortie. 
     * "description" est une chane comme "une cuisine" ou
     * "une cour de jardin".
     */
    public Room(String description) 
    {
        this.description = description;
        exits = new HashMap();
    }

    /**
     * Dfinit une sortie pour cette pice.
     */
    public void setExit(String direction, Room neighbor) 
    {
        exits.put(direction, neighbor);
    }

    /**
     * Renvoie la description de la pice
     * (telle que dfinie par le constructeur).
     */
    public String getShortDescription()
    {
        return description;
    }

    /**
     * Renvoie une description longue de cette pice, sous la forme :
     *     Vous tes dans la cuisine.
     *     Sorties : nord ouest
     */
    public String getLongDescription()
    {
        return "Vous tes dans " + description + ".\n" + getExitString();
    }

    /**
     * Renvoie une chane de caractres dcrivant les sorties de la pice, par exemple :
     * "Sorties : nord ouest."
     */
    private String getExitString()
    {
        String returnString = "Sorties :";
        Set keys = exits.keySet();
        for(Iterator iter = keys.iterator(); iter.hasNext(); )
            returnString += " " + iter.next();
        return returnString;
    }

    /**
    * Renvoie la pice atteinte si nous nous dplaons 
    * dans la direction 'direction'.
    * S'il n'y a pas de pice dans cette direction, renvoie 'null'.
    */
    public Room getExit(String direction) 
    {
        return (Room)exits.get(direction);
    }
}

