import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;

/**
 * Reprsente une grille de positions de terrain.
 * Chaque position peut contenir exactement un animal.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-09
 */
public class Field
{
    private static final Random rand = new Random();
    
    // Hauteur et largeur du terrain
    private int depth, width;
    // Zone mmoire pour les animaux
    private Object[][] field;

    /**
     * Reprsente un terrain de dimensions donnes.
     * @param depth Hauteur du terrain
     * @param width Largeur du terrain
     */
    public Field(int depth, int width)
    {
        this.depth = depth;
        this.width = width;
        field = new Object[depth][width];
    }
    
    /**
     * Vide le terrain
     */
    public void clear()
    {
        for(int row = 0; row < depth; row++) {
            for(int col = 0; col < width; col++) {
                field[row][col] = null;
            }
        }
    }
    
    /**
     * Place un animal  une position donne.
     * S'il y a dj un animal  cette position, 
     * il sera perdu.
     * @param animal Animal  placer
     * @param row Ligne de la position
     * @param col Colonne de la position
     */
    public void place(Object animal, int row, int col)
    {
        place(animal, new Location(row, col));
    }
    
    /**
     * Place un animal  une position donne.
     * S'il y a dj un animal  cette position, 
     * il sera perdu.
     * @param animal Animal  placer
     * @param location Position du placement
     */
    public void place(Object animal, Location location)
    {
        field[location.getRow()][location.getCol()] = animal;
    }
    
    /**
     * Retourne l'animal  la position donne s'il y en a un.
     * @param location Position dans le terrain
     * @return Un animal  cette position, null si aucun
     */
    public Object getObjectAt(Location location)
    {
        return getObjectAt(location.getRow(), location.getCol());
    }
    
    /**
     * Retourne l'animal  la position donne s'il y en a un.
     * @param row Ligne de la position
     * @param col Colonne de la position
     * @return Un animal  cette position, null si aucun
     */
    public Object getObjectAt(int row, int col)
    {
        return field[row][col];
    }
    
    /**
     * Gnre une position alatoire adjacente  une
     * position donne, ventuellement la mme.
     * La position retourne est toujours une position
     * valide du terrain.
     * @param location Position donne
     * @return Une position valide adjacente.
     *         Peut tre la mme que celle fournie.
     */
    public Location randomAdjacentLocation(Location location)
    {
        int row = location.getRow();
        int col = location.getCol();
        // Gnre un dcalage de -1, 0, or +1 pour les ligne et colonne courantes
        int nextRow = row + rand.nextInt(3) - 1;
        int nextCol = col + rand.nextInt(3) - 1;
        // Test de dpassement des limites
        if(nextRow < 0 || nextRow >= depth || nextCol < 0 || nextCol >= width) {
            return location;
        }
        else if(nextRow != row || nextCol != col) {
            return new Location(nextRow, nextCol);
        }
        else {
            return location;
        }
    }
    
    /**
     * Essaie de trouver une position adjacente  une position donne
     * qui soit libre. S'il n'y en a pas, retourne la position donne
     * si elle est libre. Sinon, retourne null.
     * La position retourne est dans les limites
     * du terrain.
     * @param location Position de dpart
     * @return Une position valide du terrain. Peut tre 
     *         le mme objet que la position de dpart, ou null
     *         si toutes les positions vosines sont occupes.
     */
    public Location freeAdjacentLocation(Location location)
    {
        Iterator adjacent = adjacentLocations(location);
        while(adjacent.hasNext()) {
            Location next = (Location) adjacent.next();
            if(field[next.getRow()][next.getCol()] == null) {
                return next;
            }
        }
        // tester si la position courante est libre
        if(field[location.getRow()][location.getCol()] == null) {
            return location;
        } 
        else {
            return null;
        }
    }

    /**
     * Gnre un itrateur sur une liste de positions adjacentes
     *  la position donne. Cette liste ne contient pas la position donne.
     * Toutes les positions sont valides.
     * @param location Poisition donne
     * @return Un itrateur sur les positions adjacentes
     */
    public Iterator adjacentLocations(Location location)
    {
        int row = location.getRow();
        int col = location.getCol();
        LinkedList locations = new LinkedList();
        for(int roffset = -1; roffset <= 1; roffset++) {
            int nextRow = row + roffset;
            if(nextRow >= 0 && nextRow < depth) {
                for(int coffset = -1; coffset <= 1; coffset++) {
                    int nextCol = col + coffset;
                    // Exclude invalid locations and the original location.
                    if(nextCol >= 0 && nextCol < width && (roffset != 0 || coffset != 0)) {
                        locations.add(new Location(nextRow, nextCol));
                    }
                }
            }
        }
        Collections.shuffle(locations,rand);
        return locations.iterator();
    }

    /**
     * @return La hauteur du terrain
     */
    public int getDepth()
    {
        return depth;
    }
    
    /**
     * @return La largeur du terrain
     */
    public int getWidth()
    {
        return width;
    }
}
