import java.awt.Color;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Cette classe collecte et fournit des donnes statistiques sur l'tat
 * d'un terrain. Elle est flexible: elle cre et maintient un compteur
 * pour chaque classe d'objets prsente sur le terrain.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-23
 */
public class FieldStats
{
    // Compteurs pour chacun des types d'entits (fox, rabbit, etc.) de la simulation.
    private HashMap counters;
    // Indique si les compteurs sont  jour ou non.
    private boolean countsValid;

    /**
     * Construit un objet field-statistics.
     */
    public FieldStats()
    {
        // Dfinit une collection de compteurs pour chaque type
        // d'animaux que l'on peut trouver.
        counters = new HashMap();
        countsValid = true;
    }

    /**
     * @return A Chane dcrivant quels sont les animaux prsents
     */
    public String getPopulationDetails(Field field)
    {
        StringBuffer buffer = new StringBuffer();
        if(!countsValid) {
            generateCounts(field);
        }
        Iterator keys = counters.keySet().iterator();
        while(keys.hasNext()) {
            Counter info = (Counter) counters.get(keys.next());
            if (info.getName()=="Fox") {buffer.append("Renards");} else {buffer.append("Lapins");};
            buffer.append(": ");
            buffer.append(info.getCount());
            buffer.append(' ');
        }
        return buffer.toString();
    }
    
    /**
     * Rinitialise les statistiques en cours.
     * Remet tous les compteurs  zro.
     */
    public void reset()
    {
        countsValid = false;
        Iterator keys = counters.keySet().iterator();
        while(keys.hasNext()) {
            Counter cnt = (Counter) counters.get(keys.next());
            cnt.reset();
        }
    }

    /**
     * Incrmente de un le compteur d'une classe d'animaux.
     */
    public void incrementCount(Class animalClass)
    {
        Counter cnt = (Counter) counters.get(animalClass);
        if(cnt == null) {
            // pas encore de compteur pour cette espce - en crer une
            cnt = new Counter(animalClass.getName());
            counters.put(animalClass, cnt);
        }
        cnt.increment();
    }

    /**
     * Signale que les compteurs d'animaux sont  jour.
     */
    public void countFinished()
    {
        countsValid = true;
    }

    /**
     * Dtermine si la simulation est encore "vivante",
     * c'est  dire si elle peut continuer.
     * @return true s'il y a plus d'une espce vivante.
     */
    public boolean isViable(Field field)
    {
        // nombre de compteurs non nuls
        int nonZero = 0;
        if(!countsValid) {
            generateCounts(field);
        }
        Iterator keys = counters.keySet().iterator();
        while(keys.hasNext()) {
            Counter info = (Counter) counters.get(keys.next());
            if(info.getCount() > 0) {
                nonZero++;
            }
        }
        return nonZero > 1;
    }
    
    /**
     * Calcule le nombre de renards (foxes) et lapins (rabbits).
     * Ces nombres ne sont pas mis  jour lors du placement des
     * animaux, 
     * mais seulement sur demande.
     */
    private void generateCounts(Field field)
    {
        reset();
        for(int row = 0; row < field.getDepth(); row++) {
            for(int col = 0; col < field.getWidth(); col++) {
                Object animal = field.getObjectAt(row, col);
                if(animal != null) {
                    incrementCount(animal.getClass());
                }
            }
        }
        countsValid = true;
    }
}
