import java.util.List;
import java.util.Random;

/**
 * Un modle simple de lapin (rabbit).
 * Un lapin vieillit, se dplace, se reproduit et meurt.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-11
 */
public class Rabbit
{
    // Caractristiques partages par tous les lapins (champs statiques).

    // L'ge  partir duquel un lapin peut se reproduire
    private static final int BREEDING_AGE = 5;
    // L'ge maximal d'un lapin
    private static final int MAX_AGE = 50;
    // La probabilit de reproduction d'un lapin
    private static final double BREEDING_PROBABILITY = 0.15;
    // La taille maximale d'une porte
    private static final int MAX_LITTER_SIZE = 5;
    // Un gnrateur de nombres alatoires commun pour contrler les reproductions.
    private static final Random rand = new Random();
    
    // Caractristiques individuelles (champs d'instance).
    
    // L'ge du lapin
    private int age;
    // Le lapin est vivant ou mort
    private boolean alive;
    // La position du lapin
    private Location location;

    /**
     * Cre un lapin. Un  lapin peut tre cr comme nouveau n (ge nul)
     * ou avec un ge alatoire.
     * 
     * @param randomAge Si true, le lapin aura un ge alatoire.
     */
    public Rabbit(boolean randomAge)
    {
        age = 0;
        alive = true;
        if(randomAge) {
            age = rand.nextInt(MAX_AGE);
        }
    }
    
    /**
     * Ce que fait un lapin la plupart du temps: il se dplace.
     * Il peut se reproduire ou mourir de vieillesse.
     */
    public void run(Field updatedField, List newRabbits)
    {
        incrementAge();
        if(alive) {
            int births = breed();
            for(int b = 0; b < births; b++) {
                Rabbit newRabbit = new Rabbit(false);
                newRabbits.add(newRabbit);
                Location loc = updatedField.randomAdjacentLocation(location);
                newRabbit.setLocation(loc);
                updatedField.place(newRabbit, loc);
            }
            Location newLocation = updatedField.freeAdjacentLocation(location);
            // Mettre  jour les champs seulement s'il y a une position libre
            if(newLocation != null) {
                setLocation(newLocation);
                updatedField.place(this, newLocation);
            }
            else {
                // dplacement impossible et position courante occupe
                alive = false;
            }
        }
    }
    
    /**
     * Augmente l'ge. 
     * Peut provoquer la mort (de vieillesse).
     */
    private void incrementAge()
    {
        age++;
        if(age > MAX_AGE) {
            alive = false;
        }
    }
    
    /**
     * Gnre un nombre de naissances,
     * si le lapin peut se reproduire
     * @return Le nombre de naissances (peut tre nul)
     */
    private int breed()
    {
        int births = 0;
        if(canBreed() && rand.nextDouble() <= BREEDING_PROBABILITY) {
            births = rand.nextInt(MAX_LITTER_SIZE) + 1;
        }
        return births;
    }

    /**
     * Un lapin peut se reproduire s'il a atteint l'ge adulte.
     */
    private boolean canBreed()
    {
        return age >= BREEDING_AGE;
    }
    
    /**
     * Teste si le lapin est vivant ou non.
     * @return True si le lapin est encore vivant
     */
    public boolean isAlive()
    {
        return alive;
    }

    /**
     * Signale au lapin qu'il est maintenant mort :(
     */
    public void setEaten()
    {
        alive = false;
    }
    
    /**
     * Dfinit la position de l'animal.
     * @param row Coordonne verticale (ligne) de la position
     * @param col Coordonne horizontale (colonne) de la position
     */
    public void setLocation(int row, int col)
    {
        this.location = new Location(row, col);
    }

    /**
     * Fixe la position du lapin.
     * @param location Position du lapin
     */
    public void setLocation(Location location)
    {
        this.location = location;
    }
}
