import java.io.PrintStream;
import java.util.List;
import java.util.Random;

/**
 * Un modle simple de lapin (rabbit).
 * Un lapin vieillit, se dplace, se reproduit et meurt.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002-04-11
 */
public class Rabbit extends Animal
{
    // Caractristiques partages par tous les lapins (champs statiques).

    // L'ge  partir duquel un lapin peut se reproduire
    private static final int BREEDING_AGE = 5;
    // L'ge maximal d'un lapin
    private static final int MAX_AGE = 50;
    // La probabilit de reproduction d'un lapin
    private static final double BREEDING_PROBABILITY = 0.15;
    // La taille maximale d'une porte
    private static final int MAX_LITTER_SIZE = 5;
    // Un gnrateur de nombres alatoires commun pour contrler les reproductions.
    private static final Random rand = new Random();
    
    // Caractristiques individuelles (champs d'instance).

    /**
     * Cre un lapin. Un  lapin peut tre cr comme nouveau n (ge nul)
     * ou avec un ge alatoire.
     * 
     * @param randomAge Si true, le lapin aura un ge alatoire.
     */
    public Rabbit(boolean randomAge)
    {
        super();
        if(randomAge) {
            setAge(rand.nextInt(MAX_AGE));
        }
    }
    
    /**
     * Ce que fait un lapin la plupart du temps: il se dplace.
     * Il peut se reproduire ou mourir de vieillesse.
     */
    public void act(Field currentField, Field updatedField, List newAnimals)
    {
        incrementAge();
        if(isAlive()) {
            int births = breed();
            for(int b = 0; b < births; b++) {
                Rabbit newRabbit = new Rabbit(false);
                newAnimals.add(newRabbit);
                newRabbit.setLocation(updatedField.randomAdjacentLocation(getLocation()));
                updatedField.place(newRabbit);
            }
            Location newLocation = updatedField.freeAdjacentLocation(getLocation());
            // Mettre  jour les champs seulement s'il y a une position libre
            if(newLocation != null) {
                setLocation(newLocation);
                updatedField.place(this);
            }
            else {
                // dplacement impossible et position courante occupe
                setDead();
            }

        }
    }
    
    /**
     * Augmente l'ge. 
     * Peut provoquer la mort (de vieillesse).
     */
    private void incrementAge()
    {
        setAge(getAge() + 1);
        if(getAge() > MAX_AGE) {
            setDead();
        }
    }
    
    /**
     * Gnre un nombre de naissances,
     * si le lapin peut se reproduire
     * @return Le nombre de naissances (peut tre nul)
     */
    private int breed()
    {
        int births = 0;
        if(canBreed() && rand.nextDouble() <= BREEDING_PROBABILITY) {
            births = rand.nextInt(MAX_LITTER_SIZE) + 1;
        }
        return births;
    }
    
    /**
     * @return Une reprsentation chane du lapin
     */
    public String toString()
    {
        return "Lapin, ge " + getAge();
    }

    /**
     * Un lapin peut se reproduire s'il a atteint l'ge adulte.
     */
    private boolean canBreed()
    {
        return getAge() >= BREEDING_AGE;
    }
}
