import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.TreeSet;

/**
 * Une classe pour grer les coordonnes d'un nombre quelconque de contacts.
 * Les coordonnes sont indexes par le nom et le numro de tlphone.
 * @author David J. Barnes et Michael Kolling.
 * @version 2002.06.11
 */
public class AddressBook implements Serializable
{
    // Enregistrement d'un nombre quelconque de contacts.
    private TreeMap book;
    private int numberOfEntries;

    /**
     * Initialise un carnet d'adresses vide.
     */
    public AddressBook()
    {
        book = new TreeMap();
        numberOfEntries = 0;
    }
    
    /**
     * Recherche un nom ou un numro de tlphone et renvoie
     * les coordonnes du contact correspondant.
     * @param key Le nom ou le numro  chercher.
     * @return Les informations correspondant  la cl.
     */
    public ContactDetails getDetails(String key)
    {
        return (ContactDetails) book.get(key);
    }

    /**
     * Renvoie si oui ou non la cl est utilise.
     * @param key Le nom ou le numro  chercher.
     * @return true si la cl est utilise, false sinon.
     */
    public boolean keyInUse(String key)
    {
        return book.containsKey(key);
    }

    /**
     * Ajoute un nouveau contact au carnet d'adresses.
     * @param details Les coordonnes  associer  la personne.
     */
    public void addDetails(ContactDetails details)
    {
        if(details != null) {
            book.put(details.getName(), details);
            book.put(details.getPhone(), details);
            numberOfEntries++;
        }
    }
    
    /**
     * Modifie les coordonnes stockes pour la cl donne.
     * @param oldKey Une des cls utilises pour enregistrer les informations.
     * @param details Les nouvelles informations.
     */
    public void changeDetails(String oldKey,
                              ContactDetails details)
    {
        if(keyInUse(oldKey) && details != null) {
            removeDetails(oldKey);
            addDetails(details);
        }
    }
    
    /**
     * Cherche toutes les coordonnes stockes correspondant  une cl
     * commenant par le prfixe fourni.
     * @param keyPrefix Le prfixe de recherche.
     * @return Un tableau des informations trouves.
     */
    public ContactDetails[] search(String keyPrefix)
    {
        // Construire une liste des informations qui correspondent.
        LinkedList matches = new LinkedList();
        if(keyPrefix != null) {
        // Trouver les cls suprieures ou gales au prfixe.
                SortedMap tail = book.tailMap(keyPrefix);
            Iterator it = tail.keySet().iterator();
        // Arrter lorsqu'il n'y a plus concordance.
            boolean endOfSearch = false;
            while(!endOfSearch && it.hasNext()) {
                String key = (String) it.next();
                if(key.startsWith(keyPrefix)) {
                    matches.add(book.get(key));
                }
                else {
                    endOfSearch = true;
                }
            }
        }
        ContactDetails[] results =
                        new ContactDetails[matches.size()];
        matches.toArray(results);
        return results;
    }

    /**
     * @return Le nombre d'entres actuellement 
     *         dans le carnet d'adresses.
     */
    public int getNumberOfEntries()
    {
        return numberOfEntries;
    }

    /**
     * Supprime une entre de cl donne du carnet d'adresses.
     * La cl doit tre actuellement utilise.
     * @param key Une des cls de l'entre  supprimer.
     */
    public void removeDetails(String key)
    {
        if(keyInUse(key)) {
            ContactDetails details =
                        (ContactDetails) book.get(key);
            book.remove(details.getName());
            book.remove(details.getPhone());
            numberOfEntries--;
        }
    }

    /**
     * @return Une liste de toutes les coordonnes des contacts, classes selon
     * l'ordre de tri de la classe ContactDetails.
     */
    public String listDetails()
    {
        // Comme chaque entre est stocke sous deux cls,
        // il faut construire un ensemble des coordonnes des contacts
        // pour liminer les doublons.
        StringBuffer allEntries = new StringBuffer();
        TreeSet sortedDetails = new TreeSet(book.values());
        Iterator it = sortedDetails.iterator();
        while(it.hasNext()) {
            ContactDetails details = (ContactDetails) it.next();
            allEntries.append(details);
            allEntries.append('\n');
            allEntries.append('\n');
        }
        return allEntries.toString();
    }
}
