import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

/**
 * Fournit une interface graphique pour visualiser un carnet d'adresses.
 * Diffrents panneaux donnent accs aux donnes du carnet d'adresses.
 *
 *      Un pour rechercher dans le carnet d'adresses.
 *
 *      Un pour entrer un ensemble d'informations sur plusieurs contacts.
 *      Le bouton "Ajouter" enregistre les donnes dans le carnet.
 *
 *      Un pour visualiser tout le carnet d'adresses.
 *
 * @author David J. Barnes and Michael Kolling.
 * @version 2002.05.08
 */
public class AddressBookGUI extends JFrame
{
    // Prfrences pour les dimensions de ce cadre.
    private static final int PREFERRED_WIDTH = 500;
    private static final int PREFERRED_HEIGHT = 500;
    private static final Dimension PREFERRED_SIZE =
                            new Dimension(PREFERRED_WIDTH,PREFERRED_HEIGHT);
    // Le carnet d'adresses  manipuler et  afficher.
    private AddressBook book;
    
    /**
     * Cre le cadre et ses panneaux.
     * @param book Le carnet d'adresses  grer.
     */
    public AddressBookGUI(AddressBook book)
    {
        this.book = book;
        setTitle("Address Book (Carnet d'adresses)");
        addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent ev)
            {
                setVisible(false);
            }
        });

        final Container contentPane = getContentPane();
        JTabbedPane tabbedArea = new JTabbedPane();
        tabbedArea.add("Rechercher", setupSearchArea());
        tabbedArea.add("Crer des contacts",  setupDataEntry());
        tabbedArea.add("Lister les contacts",   setupListArea());
        contentPane.add(tabbedArea);

        setSize(PREFERRED_SIZE);
    }
    
    /**
     * Visualise la fentre si elle a t ferme.
     */
    public void showWindow()
    {
        setVisible(true);
    }

    /**
     * @return La taille prfre de cette fentre.
     */
    public Dimension getPreferredSize()
    {
        return PREFERRED_SIZE;
    }

    /**
     * Dfinit le panneau d'entre de donnes.
     * @return La panneau complet.
     */
    private Container setupDataEntry()
    {
        // Dfinir la zone du nom.
        Box nameLabelArea = Box.createHorizontalBox();
        nameLabelArea.add(new JLabel("Nom", JLabel.LEFT));
        nameLabelArea.add(Box.createGlue());
        final JTextField nameField = new JTextField(50);
        Box nameArea = Box.createVerticalBox();
        nameArea.add(nameLabelArea);
        nameArea.add(nameField);

        // Dfinir la zone du numro de tlphone.
        Box phoneLabelArea = Box.createHorizontalBox();
        phoneLabelArea.add(new JLabel("Tlphone", JLabel.LEFT));
        phoneLabelArea.add(Box.createGlue());
        final JTextField phoneField = new JTextField(50);
        Box phoneArea = Box.createVerticalBox();
        phoneArea.add(phoneLabelArea);
        phoneArea.add(phoneField);
        
        // Dfinir la zone de l'adresse.
        Box addressLabelArea = Box.createHorizontalBox();
        addressLabelArea.add(new JLabel("Adresse", JLabel.LEFT));
        addressLabelArea.add(Box.createGlue());
        Box addressArea = Box.createVerticalBox();
        final JTextArea address = new JTextArea(10, 50);
        addressArea.add(addressLabelArea);
        addressArea.add(address);

        // Position des champs d'information dans le panneau.
        Box singleLineFields = Box.createVerticalBox();
        singleLineFields.add(nameArea);
        singleLineFields.add(phoneArea);
        JPanel detailsPanel = new JPanel();
        detailsPanel.setLayout(new BorderLayout());
        detailsPanel.add(singleLineFields, BorderLayout.NORTH);
        detailsPanel.add(addressArea, BorderLayout.CENTER);

        // Dfinir les boutons.
        JPanel buttonArea = new JPanel();
        JButton add = new JButton("Ajouter");
        JButton clear = new JButton("Effacer");

        // Raliser les actions lies  l'ajout d'entres.
        add.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev)
            {
                book.addDetails(
                    new ContactDetails(nameField.getText(),
                                       phoneField.getText(),
                                       address.getText()));
            }
        });

        // Effacer la zone d'entre de donnes.
        clear.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev)
            {
                nameField.setText("");
                phoneField.setText("");
                address.setText("");
            }
        });
        buttonArea.add(add);
        buttonArea.add(clear);

        // Positionner la zone des informations au dessus des boutons.
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add(detailsPanel, BorderLayout.CENTER);
        panel.add(buttonArea, BorderLayout.SOUTH);
        return panel;
    }

    /**
     * Dfinit le panneau de recherche.
     * @return Le panneau complet.
     */
    private Container setupSearchArea()
    {
        // Dfinir la zone pour entrer la chane  chercher.
        Box searchLabelArea = Box.createHorizontalBox();
        searchLabelArea.add(new JLabel("recherche", JLabel.LEFT));
        searchLabelArea.add(Box.createGlue());
        final JTextField searchField = new JTextField(50);
        Box searchArea = Box.createHorizontalBox();
        searchArea.add(searchLabelArea);
        searchArea.add(searchField);
        
        // Dfinir la zone d'affichage des rsultats.
        final JTextArea resultList = new JTextArea(10,50);
        resultList.setEditable(false);
        JScrollPane scrollArea =
                new JScrollPane(resultList,
                                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        // Tout changement du champ nom provoque une nouvelle recherche
        // dans le carnet d'adresses.
        searchField.getDocument().addDocumentListener(new DocumentListener(){
            public void changedUpdate(DocumentEvent ev)
            {
                research();
            }
            
            public void insertUpdate(DocumentEvent ev)
            {
                research();
            }
            
            public void removeUpdate(DocumentEvent ev)
            {
                research();
            }
            
            /**
             * Rechercher et prsenter les rsultats sauf si
             * la chane  trouver est vide, auquel cas la zone rsultats
             * est vide.
             */
            private void research()
            {
                String searchString = searchField.getText();
                StringBuffer buffer = new StringBuffer();
                if(searchString.length() > 0) {
                    ContactDetails[] results = book.search(searchString);
                    for(int i = 0; i < results.length; i++) {
                        buffer.append(results[i].toString());
                        buffer.append('\n');
                        buffer.append('\n');
                    }
                }
                resultList.setText(buffer.toString());
            }
        });

        JPanel listArea = new JPanel();
        listArea.setLayout(new BorderLayout());
        listArea.add(searchArea, BorderLayout.NORTH);
        listArea.add(scrollArea, BorderLayout.CENTER);

        return listArea;
    }

    /**
     * Dfinit le panneau pour le listage des informations.
     * @return Le panneau complet.
     */
    private Container setupListArea()
    {
        // Dfinir la zone d'affichage des informations.
        final JTextArea details = new JTextArea(10, 50);
        details.setEditable(false);
        JScrollPane scrollArea =
                new JScrollPane(details,
                                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        // Dfinir les boutons.
        JPanel buttonArea = new JPanel();
        JButton list = new JButton("Lister");
        JButton clear = new JButton("Effacer");
        
        // Lister toutes les entres.
        list.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev)
            {
                details.setText(book.listDetails());
            }
        });

        // Effacer la zone d'affichage.
        clear.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev)
            {
                details.setText("");
            }
        });
        buttonArea.add(list);
        buttonArea.add(clear);

        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add(scrollArea, BorderLayout.CENTER);
        panel.add(buttonArea, BorderLayout.SOUTH);

        return panel;
    }
}
