/**
 * Coordonnes: nom, adresse et numro de tlphone.
 * 
 * @author David J. Barnes and Michael Kolling.
 * @version 2002.05.08
 */
public class ContactDetails implements Comparable
{
    private String name;
    private String phone;
    private String address;

    /**
    * Initialiser les coordonnes. toutes les informations sont tronques
     * pour liminer les espaces en fin de chane.
     * @param name Le nom.
     * @param phone Le numro de tlphone.
     * @param address L'adresse.
     * @throws IllegalStateException Si le nom et le numro de tlphone sont vides.
     */
    public ContactDetails(String name, String phone, String address)
    {
        // Utiliser une chane vide si l'argument est "null".
        if(name == null) {
            name = "";
        }
        if(phone == null) {
            phone = "";
        }
        if(address == null) {
            address = "";
        }

        this.name = name.trim();
        this.phone = phone.trim();
        this.address = address.trim();

        if(this.name.length() == 0 && this.phone.length() == 0) {
            throw new IllegalStateException(
                      "Le nom ou le numro de tlphone doit tre non vide.");
        }
    }
    
    /**
     * @return Le nom.
     */
    public String getName()
    {
        return name;
    }

    /**
     * @return Le numro de tlphone.
     */
    public String getPhone()
    {
        return phone;
    }

    /**
     * @return L'adresse.
     */
    public String getAddress()
    {
        return address;
    }
    
    /**
     * Provide a content-equality test.
     */
    public boolean equals(Object other)
    {
        if(other instanceof ContactDetails) {
            ContactDetails otherDetails = (ContactDetails) other;
            return name.equals(otherDetails.getName()) &&
                    phone.equals(otherDetails.getPhone()) &&
                     address.equals(otherDetails.getAddress());
        }
        else {
            return false;
        }
    }
    
    /**
     * Compare ces coordonnes  d'autres pour raliser un tri.
     * Les champs sont ordonns par nom, tlphone et adresse.
     */
    public int compareTo(Object other)
    {
        ContactDetails otherDetails = (ContactDetails) other;
        int comparison = name.compareTo(otherDetails.getName());
        if(comparison != 0) {
            return comparison;
        }
        comparison = phone.compareTo(otherDetails.getPhone());
        if(comparison != 0) {
            return comparison;
        }
        return address.compareTo(otherDetails.getAddress());
    }

    /**
     * @return Une chane multiligne contenant le nom, le tlphone et l'adresse.
     */
    public String toString()
    {
        return name + "\n" + phone + "\n" + address;
    }
}
