import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
    
/**
 * Une collection d'lments dans la ville.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class City
{
    private List items;
    private int width;
    private int height;
    
    private static final int DEFAULT_WIDTH = 35;
    private static final int DEFAULT_HEIGHT = 35;

    /**
     * Constructeur pour les objets de la classe City
     * @param width La largeur de la ville.
     * @param height La hauteur de la ville.
     */
    public City(int width, int height)
    {
        if(width < 1) {
            throw new IllegalArgumentException(
                        "La largeur doit tre positive: " +
                        width);
        }
        if(height < 1) {
            throw new IllegalArgumentException(
                        "La hauteur doit tre positive: " +
                        height);
        }
        this.width = width;
        this.height = height;
        items = new LinkedList();
    }
    
    /**
     * Cre une ville avec les dimensions par dfaut.
     */
    public City()
    {
        this(DEFAULT_WIDTH, DEFAULT_HEIGHT);
    }
    
    /**
     * @return Un itrateur sur les lments.
     */
    public Iterator getItems()
    {
        return items.iterator();
    }

    /**
     * Ajoute l'lment donn  la collection d'lments de la ville.
     * @param item L'lment  ajouter.
     */
    public void addItem(Object item)
    {
        if(items.contains(item)) {
            throw new IllegalArgumentException(
                item + " dj enregistr dans la ville.");
        }
        items.add(item);
    }

    /**
     * Supprime l'lment donn de la collection d'lments de la ville.
     * @param item L'lment  supprimer.
     */
    public void removeItem(Object item)
    {
        if(!items.remove(item)) {
            throw new IllegalArgumentException(
                        item + " n'est pas dans la ville.");
        }
    }
        
    /**
     * @return Une reprsentation chane de la ville.
     */
    public String toString()
    {
        return "Dimensions de la ville: " + width + " par " + height;
    }
    
    /**
     * @return La largeur.
     */
    public int getWidth()
    {
        return width;
    }
    
    /**
     * @return La hauteur.
     */
    public int getHeight()
    {
        return height;
    }
}
