import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.awt.*;
import javax.swing.*;
    
/**
 * Fournit une vue des vhicules et des passagers de la ville.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class CityGUI extends JFrame implements Actor
{
    private City city;
    private CityView cityView;
    
    /**
     * Constructeur d'objets de la classe CityGUI
     * @param city La ville dont on veut afficher la vue.
     */
    public CityGUI(City city)
    {
        this.city = city;
        cityView = new CityView(city.getWidth(), city.getHeight());
        getContentPane().add(cityView);
        setTitle("Taxiville");
        setSize(800, 800);
        setVisible(true);
    }
    
    /**
     * Affiche l'tat courant de la ville.
     */
    public void act()
    {
        cityView.preparePaint();
        Iterator items = city.getItems();
        while(items.hasNext()) {
            Object obj = items.next();
            if(obj instanceof DrawableItem){
                DrawableItem item = (DrawableItem) obj;
                Location location = item.getLocation();
                cityView.drawImage(location.getX(), location.getY(), item.getImage());
            }
        }
        cityView.repaint();
    }
    
    /**
     * Forunit une vue graphique d'une ville rectangulaire. C'est une
     * classe imbrique (une classe dfinie dans une classe) qui
     * dfinit un composant adapt  l'interface utilisateur.
     * Ce composant affiche la ville.
     * Ce code est assez compliqu. Vous pouvez l'ignorer
     * pour vos projets si vous prfrez.
     */
    private class CityView extends JPanel
    {
        private final int VIEW_SCALING_FACTOR = 6;

        private int cityWidth, cityHeight;
        private int xScale, yScale;
        private Dimension size;
        private Graphics g;
        private Image cityImage;

        /**
         * Create a new CityView component.
         */
        public CityView(int width, int height)
        {
            cityWidth = width;
            cityHeight = height;
            setBackground(Color.white);
            size = new Dimension(0, 0);
        }

        /**
         * Indique au gestionnaire graphique quelle taille nous voulons.
         */
        public Dimension getPreferredSize()
        {
            return new Dimension(cityWidth * VIEW_SCALING_FACTOR,
                                 cityHeight * VIEW_SCALING_FACTOR);
        }
        
        /**
         * Prpare un nouveau cycle d'afichage. Comme le composant
         * peut tre redimensionn, recalculer le facteur d'chelle.
         */
        public void preparePaint()
        {
            if(!size.equals(getSize())) {  // si la taille a chang...
                size = getSize();
                cityImage = cityView.createImage(size.width, size.height);
                g = cityImage.getGraphics();

                xScale = size.width / cityWidth;
                if(xScale < 1) {
                    xScale = VIEW_SCALING_FACTOR;
                }
                yScale = size.height / cityHeight;
                if(yScale < 1) {
                    yScale = VIEW_SCALING_FACTOR;
                }
            }
            g.setColor(Color.white);
            g.fillRect(0, 0, size.width, size.height);
            g.setColor(Color.gray);
            for(int i = 0, x = 0; x < size.width; i++, x = i * xScale) {
                g.drawLine(x, 0, x, size.height - 1);
            }
            for(int i = 0, y = 0; y < size.height; i++, y = i * yScale) {
                g.drawLine(0, y, size.width - 1, y);
            }
        }
        
        /**
         * Dessine l'image d'un lment donn.
         */
        public void drawImage(int x, int y, Image image)
        {
            g.drawImage(image, x * xScale + 1, y * yScale + 1,
                        xScale - 1, yScale - 1, this);
        }

        /**
         * Le composant "vue de la ville" doit tre raffich. Copie
         * la vue interne sur l'cran.
         */
        public void paintComponent(Graphics g)
        {
            if(cityImage != null) {
                g.drawImage(cityImage, 0, 0, null);
            }
        }
    }
}
