import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

/**
 * Gnre des passagers priodiquement.
 * Comptabiliser le nombre de passagers
 * qui n'ont pu avoir de vhicule.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class PassengerSource implements Actor
{
    private City city;
    private TaxiCompany company;
    private Random rand;
    private static final double CREATION_PROBABILITY = 0.06;
    private int missedPickups;

    /**
     * Constructeur d'objets de la classe PassengerSource.
     * @param city La ville. Doit tre non 'null'.
     * @param company La compagnie  utiliser. Doit tre non 'null'.
     * @throws NullPointerException si la compagnie ou la ville est 'null'.
     */
    public PassengerSource(City city, TaxiCompany company)
    {
        if(city == null) {
            throw new NullPointerException("ville");
        }
        if(company == null) {
            throw new NullPointerException("compagnie");
        }
        this.city = city;
        this.company = company;
        // Utilise une semence fixe pour avoir des effets reproductibles.
        // A changer pour produire des effets alatoires.
        rand = new Random(12345);
        missedPickups = 0;
    }

    /**
     * Gnre un nouveau passager au hasard.
     * Comptabilise les demandes qui ont chou.
     */
    public void act()
    {
        if(rand.nextDouble() <= CREATION_PROBABILITY) {
            Passenger passenger = createPassenger();
            if(company.requestPickup(passenger)) {
                city.addItem(passenger);
            }
            else {
                missedPickups++;
            }
        }
    }

    /**
     * @return Le nombre de passagers dont la demande
     *         n'a pas abouti.
     */
    public int getMissedPickups()
    {
        return missedPickups;
    }

    /**
     * Cre un nouveau passager avec des lieux de dpart et d'arrive
     * distincts.
     * @return Le passager cr.
     */
    private Passenger createPassenger()
    {
        int cityWidth = city.getWidth();
        int cityHeight = city.getHeight();

        Location pickupLocation =
                    new Location(rand.nextInt(cityWidth),
                                 rand.nextInt(cityHeight));
        Location destination;
        do{
            destination =
                    new Location(rand.nextInt(cityWidth),
                                 rand.nextInt(cityHeight));
        } while(pickupLocation.equals(destination));
        return new Passenger(pickupLocation, destination);
    }
}
