import java.awt.Image;
import javax.swing.ImageIcon;

/**
 * Un taxi peut transporter un seul passager.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class Taxi extends Vehicle implements DrawableItem
{
    private Passenger passenger;
    // Contient des images diffrentes selon que le taxi est vide
    // ou plein.
    private Image emptyImage, passengerImage;
    
    /**
     * Constructeur des objets de la classe Taxi
     * @param company La compagnie de taxis. Doit tre non 'null'.
     * @param location le point de dpart du vhicule. Doit tre non 'null'.
     * @throws NullPointerException Si 'company' ou 'location' est 'null'.
     */
    public Taxi(TaxiCompany company, Location location)
    {
        super(company, location);
    // Charger les deux images.
        emptyImage = new ImageIcon(getClass().getResource(
                                "images/taxi.jpg")).getImage();

        passengerImage = new ImageIcon(getClass().getResource(
                                "images/taxi+person.jpg")).getImage();
    }
    
    /**
     * Avancer vers la destination s'il y en a une.
     * Sinon, enregistrer que l'on est inactif.
     */
    public void act()
    {
        Location target = getTargetLocation();
        if(target != null) {
            // Trouver o aller maintenant.
            Location next = getLocation().nextLocation(target);
            setLocation(next);
            if(next.equals(target)) {
                if(passenger != null) {
                    notifyPassengerArrival(passenger);
                    offloadPassenger();
                }
                else {
                    notifyPickupArrival();
                }
            }
        }
        else {
            incrementIdleCount();
        }
    }

    /**
     * @return Si oui ou non le taxi est libre.
     */
    public boolean isFree()
    {
        return getTargetLocation() == null && passenger == null;
    }
    
    /**
     * Reoit un lieu de prise en charge qui
     * devient la destination.
     * @location Le lieu de prise en charge.
     */
    public void setPickupLocation(Location location)
    {
        setTargetLocation(location);
    }
    
    /**
     * Reoit un passager.
     * Dfinit sa destination comme le lieu  atteindre.
     * @param passenger Le passager.
     */
    public void pickup(Passenger passenger)
    {
        this.passenger = passenger;
        setTargetLocation(passenger.getDestination());
    }

    /**
     * Offload the passenger.
     */
    public void offloadPassenger()
    {
        passenger = null;
        clearTargetLocation();
    }
    
    /**
     * Renvoie une image dcrivant l'tat:
     * vide ou transportant un passager.
     */
    public Image getImage()
    {
        if(passenger != null) {
            return passengerImage;
        }
        else {
            return emptyImage;
        }
    }

    /**
     * @return une chane reprsentant le taxi.
     */
    public String toString()
    {
        return "Taxi en " + getLocation();
    }
}
