import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.Random;

/**
 * Modlise le fonctionnement d'une compagnie de taxis avec diffrents vhicules.
 * Cette version gre seulement des taxis.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class TaxiCompany  
{
    // Les vhicules grs par la compagnie.
    private List vehicles;
    private City city;
    // Les associations entre les vhicules et les lieux o prendre
    // les passagers.
    private Map assignments;

    private static final int NUMBER_OF_TAXIS = 3;

    /**
     * @param city La ville.
     */
    public TaxiCompany(City city)
    {
        this.city = city;
        vehicles = new LinkedList();
        assignments = new HashMap();
        setupVehicles();
    }

    /**
     * Demande une prise en charge pour le passager donn.
     * @param passenger Le passager demandant une course.
     * @return Si oui ou non un vhicule est libre.
     */
    public boolean requestPickup(Passenger passenger)
    {
        Vehicle vehicle = scheduleVehicle();
        if(vehicle != null) {
            assignments.put(vehicle, passenger);
            vehicle.setPickupLocation(passenger.getPickupLocation());
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Un vhicule est arriv au lieu de prise en charge.
     * @param Le vhicule au point de chargement.
     * @throws MissingPassengerException S'il n'y a pas de passager en attente.
     */
    public void arrivedAtPickup(Vehicle vehicle)
    {
        Passenger passenger = (Passenger) assignments.remove(vehicle);
        if(passenger == null) {
            throw new MissingPassengerException(vehicle);
        }
        city.removeItem(passenger);
        vehicle.pickup(passenger);
    }
    
    /**
     * Un vhicule est arriv  destination.
     * @param Le vhicule  destination.
     * @param Le passager dbarqu.
     */
    public void arrivedAtDestination(Vehicle vehicle,
                                     Passenger passenger)
    {
    }
    
    /**
     * @return La liste des vhicules.
     */
    public List getVehicles()
    {
        return vehicles;
    }
    
    /**
     * Trouve un vhicule libre si possible.
     * @return Un vhicule libre, 'null' s'il n'y en a pas.
     */
    private Vehicle scheduleVehicle()
    {
        Iterator it = vehicles.iterator();
        while(it.hasNext()) {
            Vehicle vehicle = (Vehicle) it.next();
            if(vehicle.isFree()) {
                return vehicle;
            }
        }
        return null;
    }

    /**
     * Dfinit les vhicules de cette compagnie. Le nombre optimal de vhicules
     * doit tre dtermin par analyse des donnes
     * produites par la simulation.
     *
     * Les vhicules sont placs  des positions alatoires.
     */
    private void setupVehicles()
    {
        int cityWidth = city.getWidth();
        int cityHeight = city.getHeight();
        // Used a fixed random seed for predictable behavior.
        // Use different seeds for less predictable behavior.
        Random rand = new Random(12345);

        // Create the taxis.
        for(int i = 0; i < NUMBER_OF_TAXIS; i++){
            Taxi taxi =
                new Taxi(this,
                         new Location(rand.nextInt(cityWidth),
                                      rand.nextInt(cityHeight)));
            vehicles.add(taxi);
            city.addItem(taxi);
        }
   }
}
