/**
 * Rsum des informations sur un vhicule.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.06.20
 */
public abstract class Vehicle
{
    private TaxiCompany company;
    // O est le vhicule.
    private Location location;
    // O se rend le vhicule.
    private Location targetLocation;
    
    /**
     * Constructeur de la classe 'Vehicle'
     * @param company La compagnie de taxis. Doit tre non 'null'.
     * @param location Le lieu de dpart du vhicule. Doit tre non 'null'.
     * @throws NullPointerException Si la compagnie ou le lieu est 'null'.
     */
    public Vehicle(TaxiCompany company, Location location)
    {
        if(company == null) {
            throw new NullPointerException("compagnie");
        }
        if(location == null) {
            throw new NullPointerException("position");
        }
        this.company = company;
        this.location = location;
        targetLocation = null;
    }
    
    /**
     * Informe la compagnie de notre arrive au point de prise en charge.
     */
    public void notifyPickupArrival()
    {
        company.arrivedAtPickup(this);
    }
    
    /**
     * Informe la compagnie de notre arrive au lieu
     * de destination du passager.
     * @param passenger Le passager qui est arriv.
     */
    public void notifyPassengerArrival(Passenger passenger)
    {
        company.arrivedAtDestination(this, passenger);
    }
    
    /**
     * Reoit un lieu de chargement.
     * La gestion dpend du type de vhicule.
     * @param location Le lieu de chargement.
     */
    public abstract void setPickupLocation(Location location);
    
    /**
     * Reoit un passager.
     * La gestion dpend du type de vhicule.
     * @param passenger Le passager  charger.
     */
    public abstract void pickup(Passenger passenger);
    
    /**
     * @return le fait que ce vhicule est libre ou pas.
     */
    public abstract boolean isFree();
    
    /**
     * Dpose les passagers dont la destination est
     * le lieu courant.
     */
    public abstract void offloadPassenger();
    
    /**
     * @return O est actuellement le vhicule.
     */
    public Location getLocation()
    {
        return location;
    }
    
    /**
     * Dfinit la position courante.
     * @param location O l'on est. Doit tre non 'null'.
     * @throws NullPointerException Si 'location' est 'null'.
     */
    public void setLocation(Location location)
    {
        if(location != null) {
            this.location = location;
        }
        else {
            throw new NullPointerException();
        }
    }
    
    /**
     * @return Quelle est la destination actuelle du vhicule, ou 'null'
     *         s'il est inactif.
     */
    public Location getTargetLocation()
    {
        return targetLocation;
    }
    
    /**
     * Dfinit la destination.
     * @param location O se rendre. Doit tre non 'null'.
     * @throws NullPointerException Si 'location' est 'null'.
     */
    public void setTargetLocation(Location location)
    {
        if(location != null) {
            targetLocation = location;
        }
        else {
            throw new NullPointerException();
        }
    }
    
    /**
     * Met  vide la destination.
     */
    public void clearTargetLocation()
    {
        targetLocation = null;
    }
}
