/**
 * Modlise une position dans la ville.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class Location
{
    private int x;
    private int y;

    /**
     * Modlise une position dans la ville.
     * @param x La coordonne x. Doit tre positive.
     * @param y La coordonne y. Doit tre positive.
     * @throws IllegalArgumentException Si une coordonne est ngative.
     */
    public Location(int x, int y)
    {
        if(x < 0) {
            throw new IllegalArgumentException(
                        "Coordonne x ngative: " + x);
        }
        if(y < 0) {
            throw new IllegalArgumentException(
                        "Coordonne y ngative: " + y);
        }
        this.x = x;
        this.y = y;
    }
    
    /**
     * Gnre la position suivante  visiter pour
     * atteindre la destionation.
     * @param destination O nous voulons aller.
     * @return Une position en ligne droite depuis la position courante
     *         jusqu' la destination.
     */
    public Location nextLocation(Location destination)
    {
        int destX = destination.getX();
        int destY = destination.getY();
        int offsetX = x < destX ? 1 : x > destX ? -1 : 0;
        int offsetY = y < destY ? 1 : y > destY ? -1 : 0;
        if(offsetX != 0 || offsetY != 0) {
            return new Location(x + offsetX, y + offsetY);
        }
        else {
            return destination;
        }
    }

    /**
     * Dtermine le nombre de mouvements ncessaires pour aller
     * d'ici  la destination.
     * @param destination La destination demande.
     * @return Le nombre de pas de mouvement.
     */
    public int distance(Location destination)
    {
        int xDist = Math.abs(destination.getX() - x);
        int yDist = Math.abs(destination.getY() - y);
        return Math.max(xDist, yDist);
    }
    
    /**
     * Implante l'galit de contenu pour les positions.
     * @return true si la position est gale  celle de 'other'
     *         false sinon.
     */
    public boolean equals(Object other)
    {
        if(other instanceof Location) {
            Location otherLocation = (Location) other;
            return x == otherLocation.getX() &&
                   y == otherLocation.getY();
        }
        else {
            return false;
        }
    }
    
    /**
     * @return Une reprsentation de la position.
     */
    public String toString()
    {
        return "position " + x + "," + y;
    }

    /**
     * @return La coordonne x.
     */
    public int getX()
    {
        return x;
    }

    /**
     * @return La coordonne y.
     */
    public int getY()
    {
        return y;
    }
}
