import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;

/**
 * Modlise le fonctionnement d'une compagnie de taxis avec diffrents vhicules.
 * Cette version gre seulement un seul taxi.
 * 
 * @author David J. Barnes and Michael Kolling
 * @version 2002.07.02
 */
public class TaxiCompany  
{
    // Les vhicules grs par la compagnie.
    private List vehicles;
    // Les associations entre les vhicules et les lieux o prendre
    // les passagers.
    private Map assignments;

    /**
     * Constructeur pour les objets de la classe TaxiCompany
     */
    public TaxiCompany()
    {
        vehicles = new LinkedList();
        assignments = new HashMap();
    }

    /**
     * Demande une prise en charge pour le passager donn.
     * @param passenger Le passager demandant une course.
     * @return Si oui ou non un vhicule est libre.
     */
    public boolean requestPickup(Passenger passenger)
    {
        Vehicle vehicle = scheduleVehicle();
        if(vehicle != null) {
            assignments.put(vehicle, passenger);
            vehicle.setPickupLocation(passenger.getPickupLocation());
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Un vhicule est arriv au lieu de prise en charge.
     * @param Le vhicule au point de chargement.
     * @throws MissingPassengerException S'il n'y a pas de passager en attente.
     */
    public void arrivedAtPickup(Vehicle vehicle)
    {
        Passenger passenger = (Passenger) assignments.remove(vehicle);
        if(passenger == null) {
            throw new MissingPassengerException(vehicle);
        }
        System.out.println(vehicle + " prend " + passenger);
        vehicle.pickup(passenger);
    }
    
    /**
     * Un vhicule est arriv  destination.
     * @param Le vhicule  destination.
     * @param Le passager dbarqu.
     */
    public void arrivedAtDestination(Vehicle vehicle,
                                     Passenger passenger)
    {
        System.out.println(vehicle + " dbarque " + passenger);
    }
    
    /**
     * @return The list of vehicles.
     */
    public List getVehicles()
    {
        return vehicles;
    }
    
    /**
     * Trouve un vhicule libre si possible.
     * @return Un vhicule libre, 'null' s'il n'y en a pas.
     */
    private Vehicle scheduleVehicle()
    {
        Iterator it = vehicles.iterator();
        while(it.hasNext()) {
            Vehicle vehicle = (Vehicle) it.next();
            if(vehicle.isFree()) {
                return vehicle;
            }
        }
        return null;
    }
}
