import java.util.HashMap;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

/**
 * La classe responder reprsente un objet gnrateur de rponses.
 * Elle permet de gnrer une rponse automatique, base sur une saisie spcifique.
 * La saisie est prsente au responder sous la forme d'un ensemble de mots et, en fonction de ces
 * mots, le responder gnre une chane qui reprsente la rponse.
 *
 * En interne, le reponder utilise un HashMap pour associer les mots aux chanes de
 * rponse et  la liste des rponses par dfaut. Si l'un des mots saisis est trouv
 * dans le HashMap, la rponse correspondante est renvoye. Lorsqu'aucun des mots
 * saisis n'est reconnu, l'une des rponses par dfaut est choisie alatoirement.
 * 
 * @version    1.1  (21.Jan.2002)
 * @author     Michael Kolling et David J. Barnes
 */
public class Responder
{
    private HashMap responseMap;          // utilis pour faire correspondre des mots-cl  des rponses
    private ArrayList defaultResponses;   // rponses par dfaut utilises si aucun mot n'est reconnu 
    private Random randomGenerator;

    /**
     * Construit un Responder
     */
    public Responder()
    {
        responseMap = new HashMap();
        defaultResponses = new ArrayList();
        fillResponseMap();
        fillDefaultResponses();
        randomGenerator = new Random();
    }

    /**
     * Gnre une rponse d'un ensemble donn de mots saisis.
     * 
     * @param words  Un ensemble de mots saisis par l'utilisateur
     * @return       Une chane qui doit tre affiche en rponse
     */
    public String generateResponse(HashSet words)
    {
        Iterator it = words.iterator();
        while(it.hasNext()) {
            String word = (String) it.next();
            String response = (String) responseMap.get(word);
            if(response != null) {
                return response;
            }
        }
        // Si nous arrivons ici, aucun des mots saisis n'a t reconnu.
        // Dans ce cas, nous choisissons une rponse par dfaut (ce que nous disons lorsque
        // nous ne savons pas quoi dire...)

        return pickDefaultResponse();
    }

    /**
     * Entre tous les mots-cl connus et leurs rponses associes
     * dans notre correspondance de rponses.
     */
    private void fillResponseMap()
    {
        responseMap.put("plante", 
                        "Eh bien, a ne plante pas sur notre systme. Cela doit tre li\n" +
                        "avec votre systme. Dites-m'en plus au sujet de votre configuration.");
        responseMap.put("planter", 
                        "Eh bien, a ne plante pas sur notre systme. Cela doit tre li\n" +
                        "avec votre systme. Dites-m'en plus au sujet de votre configuration.");
        responseMap.put("lent", 
                        "Je pense que c'est li avec votre matriel. La mise  niveau de votre processeur\n" +
                        "devrait rsoudre tous vos problmes de performances. Avez-vous un problme avec\n" +
                        "notre logiciel ?");
        responseMap.put("performances", 
                        "Les performances taient assez bonnes lors de nos tests. D'autres procdures\n" +
                        "s'excutent-elles en arrire-plan ?");
        responseMap.put("bogue", 
                        "Vous savez, tous les logiciels ont des bogues. Mais nos ingnieurs logiciels\n" +
                        "travaillent pour les rsoudre. Pouvez-vous dcrire le problme\n" +
                        "en dtails ?");
        responseMap.put("bug", 
                        "Vous savez, tous les logiciels ont des bogues. Mais nos ingnieurs logiciels\n" +
                        "travaillent pour les rsoudre. Pouvez-vous dcrire le problme\n" +
                        "en dtails ?");
        responseMap.put("windows", 
                        "Ceci est un bogue connu li au systme d'exploitation Windows. Veuillez\n" +
                        "le signaler  Microsoft. Nous ne pouvons rien faire  cela.");
        responseMap.put("macintosh", 
                        "Ceci est un bogue connu li au systme d'exploitation Mac. Veuillez\n" +
                        "le signaler  Apple. Nous ne pouvons rien faire  cela.");
        responseMap.put("cher", 
                        "Le prix de notre produit est assez concurrentiel. Avez-vous tudi\n" +
                        "et vraiment compar les fonctionnalits que nous proposons ?");
        responseMap.put("installation", 
                        "L'installation est assez simple. Nous avons beaucoup\n" +
                        "d'assistants qui font le travail pour vous. Avez-vous lu les instructions\n" +
                        "d'installation ?");
        responseMap.put("mmoire", 
                        "Si vous lisez attentivement la configuration requise, vous verrez que la\n" +
                        "mmoire spcifie doit tre de 1,5 Go. Vous devriez\n" +
                        "mettre  jour votre mmoire. Vouliez-vous savoir autre chose ?");
        responseMap.put("linux", 
                        "Nous prenons trs au srieux la prise en charge de Linux. Certains problmes subsistent.\n" +
                        "La plupart sont lis  une incompatibilit des versions glibc. Pouvez-vous tre un peut\n" +
                        "plus prcis ?");
        responseMap.put("bluej", 
                        "Ahhh, BlueJ, oui. Nous avons essay d'acheter ces mecs depuis longtemps, mais\n" +
                        "ils ne veulent pas vendre... Ce sont des gens borns. Nous ne pouvons rien\n" +
                        "faire  cela, j'en ai peur.");
    }

    /**
     * Construit une liste des rponses par dfaut  partir de laquelle choisir
     * si nous ne savons pas quoi dire.
     */
    private void fillDefaultResponses()
    {
        defaultResponses.add("Cela semble trange. Pourriez-vous dcrire ce problme plus prcisment ?");
        defaultResponses.add("Aucun autre client ne s'est jamais plaint de cela. \n" +
                             "Quelle est votre configuration systme ?");
        defaultResponses.add("Cela est intressant. Dites-m'en plus...");
        defaultResponses.add("J'ai besoin d'un peu plus d'informations  ce sujet.");
        defaultResponses.add("Avez-vous vrif que vous n'avez pas de conflit de dll ?");
        defaultResponses.add("Ceci est expliqu dans le manuel. Avez-vous lu le manuel ?");
        defaultResponses.add("Votre description est un peu confuse. Y a-t-il un expert\n" +
                             "prs de vous qui pourrait dcrire ceci plus prcisment ?");
        defaultResponses.add("Ce n'est pas un bogue, c'est une fonctionnalit !");
        defaultResponses.add("Pourriez-vous prciser ?");
    }

    /**
     * Slectionne alatoirement l'une des rponses par dfaut.
     * @return     Une rponse alatoire par dfaut
     */
    private String pickDefaultResponse()
    {
        // Choisit un numro alatoire pour l'index dans la liste de rponses par dfaut.
        // Le nombre sera compris entre 0 (inclus) et la taille de la liste (exclue).
        int index = randomGenerator.nextInt(defaultResponses.size());
        return (String) defaultResponses.get(index);
    }
}
